classdef axisProps < handle
  properties
    % handles to axes to plot on
    ax1
    ax2
    ax3
    layout
    toolBar
    
    % settings for plotting
    data
    titles
    subtitles
    ylabels
    xlabels
    colors
    plotType
    linewidth
    XLim
    YLim
    
    lineStyle
    
    
    numDatasets
    numAxes

    
    %text area handle
    textArea

    
    lineStyleForClusteredSignalPlots = ':';
    
    currentTab
    
    
    timeVector
    
    listeners
    
    captureImageMode = false;
    smoothKmeans
    darkGreenRedFusha = {[0.3137    0.6706    0.0588], [0.6353    0.0784    0.1843], [1.0000    0.0745    0.6510]}; %darkGreen,darkRed, fusha
    colorOfRemovedParts = [0.6 0.8 1];
    ckValues
    addCKEdges
  end
  properties (SetObservable)
    handleToMol
    filterInputs

    xUnits
    FrameRate
    updateTextAreaFlag
  end
  
  properties (Dependent)
    %related to how to plot
    currentAnalysisStep;

  end
  events

    updateAxisLimitsInAxProps
    tabSelectionChanged
    FrameRateChanged
    molNumChanged
    addCKEdgesToPlot
  end
  methods
    function axProps = axisProps(handleToApp, handleToMol)
      axProps.handleToMol = handleToMol;
      % get handles to objects from main app
      axProps.currentTab = handleToApp.TabGroup.SelectedTab.Tag;
      axProps.textArea = handleToApp.MolStatsTextArea;
      axProps.updateTextAreaFlag = handleToApp.UseRedFramesButton.Value;
      axProps.addCKEdges = handleToApp.ShowonallplotsCheckBox.Value;
      
      % get properties specific to kmeans
      smoothKmeansProps = handleToApp.kmeansSmoothingProps;
      appComponents = handleToApp.kmeansSmoothingAppComponents;
      for i = 1:size(smoothKmeansProps,2)
        axProps.smoothKmeans.(smoothKmeansProps{i}) = handleToApp.(appComponents{i}).Value;
      end
      axProps.smoothKmeans.plotClusteredSignalTF = false;
      % properties specific to data filtering
      axProps.filterInputs = handleToApp.filterInputs;
%       bleachProps = handleToApp.bleachProps;
%       appComponents = handleToApp.bleachAppComponents;
%       for i = 1:size(bleachProps,2)
%         axProps.bleachSettings.(bleachProps{i}) = handleToApp.(appComponents{i}).Value;
%       end
      % other properties related to plotting
      axProps.xUnits = handleToApp.xaxisunitsDropDown.Value;
      axProps.FrameRate = handleToApp.FrameratemsEditField.Value; %in miliseconds
      
      %set default limits for plots
      tabNames = {'kmeansTab', 'filterTab'};
      nAxes = [3, 3];
      nFrames = axProps.handleToMol.numFrames;
      for i = 1:size(tabNames,2)
        for n = 1:nAxes(i)
          axField = ['ax', num2str(n)];
          axProps.YLim.(axField).(tabNames{i}).default = [-inf, inf];
          axProps.XLim.(axField).(tabNames{i}).default = [1, nFrames];
          if n == 2
            axProps.YLim.(axField).(tabNames{i}).default = [-0.25, 1.25];
          end
          axProps.YLim.(axField).(tabNames{i}).limits = axProps.YLim.(axField).(tabNames{i}).default;
          axProps.XLim.(axField).(tabNames{i}).limits = axProps.XLim.(axField).(tabNames{i}).default;
        end
      end
      axProps.timeVector = 1:nFrames;
      
      
      % set up graphing layout
      axProps.layout = tiledlayout(handleToApp.TracePlotPanel,6,3,'TileSpacing','none', "Padding","none");
      plotTitle = title(axProps.layout, 'loading');
      plotTitle.Interpreter = 'none';
      
      if axProps.filterInputs.thirdAxForPlot == true && ismember(axProps.currentTab, {'filterTab'})
        axProps.numAxes = 3;
        for i = 1:axProps.numAxes
          axName = ['ax', num2str(i)];
          axProps.(axName) = nexttile(axProps.layout, [2,3]);
          set(axProps.(axName), 'Interactions', []);
        end
      else
        axProps.numAxes = 2;
        for i = 1:axProps.numAxes
          axName = ['ax', num2str(i)];
          axProps.(axName) = nexttile(axProps.layout, [3,3]);
          set(axProps.(axName), 'Interactions', []);
        end
      end

      setUpToolbar(axProps);
      axProps.plot;
      
      % property listeners
      axProps.listeners.analysisStep = addlistener(axProps.handleToMol,'currentAnalysisStep','PostSet',@(src,evnt) axProps.plot);
      axProps.listeners.tab = addlistener(axProps, 'tabSelectionChanged', @(src, evnt) axProps.selectedTabChanged(evnt));
      axProps.listeners.molNum = addlistener(handleToApp, 'moleculeNumber', 'PostSet', @(src,evnt)axProps.updateMolNum(evnt.AffectedObject.moleculeNumber,...
        evnt.AffectedObject.moleculeArray));
      %axProps.listeners.dataFilter = addlistener(handleToApp, 'filterInputs', 'PostSet', @(src,evnt)axProps.updateFilterInputs(evnt.AffectedObject.filterInputs));
      axProps.listeners.textUpdate = addlistener(handleToApp.UseRedFramesButton, 'ValueChanged', @(src,evnt) set(axProps.updateTextAreaFlag, evnt.Value));
      %axProps.listeners.filterSettings = addlistener(handleToApp, 'dataFilterInputs', 'PostSet', @(src,evnt) axProps.plot);
     axProps.ckValues = {handleToApp.WindowSizeSpinner.Value, handleToApp.MinpeakheightSpinner.Value, ...
        handleToApp.PercentileEditField.Value, handleToApp.SeparationSpinner.Value, handleToApp.ttestfilterCheckBox.Value};
      %axProps.listeners.CK = addlistener(axProps.ckValues, 'ValueChanged', @(s,e) axProps.updateCKValues(e.AffectedObject));

    end
        %%%%%% Pengning's CK edge detection code: %%%%%%%%%%%%%%%
    function [CKtransitions, CKtransindex] = CKedgedetection(~, FRETinten, window, minpeakheight, percentile, separation, ttestswitch)
      % NOTE: This program has all the pieces to calculate transitions in the donor and acceptor traces. However, the t-test is
      % only set up for the FRET data, and the unused code is commented out.
      
      lendata = length(FRETinten);
      lenP = length(percentile);
      
      % Output matrices.
      frtbapred(lendata) = 0; frtfwpred(lendata) = 0;
      
      % Looping through the timepoints
      for tpts = 1 : lendata
        
        % If we're at a data point within the largest window size from the start or end of the data...
        if tpts-window-1 >0 && tpts+window+1<= lendata
          
          frtbapred(tpts) = std(FRETinten(tpts-window-1:tpts-1))^2;
          frtfwpred(tpts) = std(FRETinten(tpts+1:tpts+window+1))^2;
        end
      end
      
      % Finding locations where the predictor window's standard deviation is highest (which should happen when a
      % transition occurs within that window). StDevs higher than the input percentiles are kept.
      fbloc = zeros(ceil(lendata/separation),lenP); ffloc = zeros(ceil(lendata/separation),lenP);
      
      for PCT = 1:lenP
        [~, loc5] = findpeaks(frtbapred,'minpeakheight',minpeakheight,'minpeakdistance',separation);
        [~, loc6] = findpeaks(frtfwpred,'minpeakheight',minpeakheight,'minpeakdistance',separation);
        for t = 1:length(loc5)
          fbloc(t,PCT) = loc5(t);
        end
        for t = 1:length(loc6)
          ffloc(t,PCT) = loc6(t);
        end
      end
      
      % Fixing the windows for the offset between forward and backward windows.
      offset = window/2+1;
      for PCT = 1:lenP
        for i = 1:length(fbloc)
          if fbloc(i,PCT) ~= 0
            fbloc(i,PCT) = fbloc(i,PCT)-offset;
          end
        end
        for i = 1:length(ffloc)
          if ffloc(i,PCT) ~= 0
            ffloc(i,PCT) = ffloc(i,PCT)+offset;
          end
        end
      end
      
      ftransloc = [fbloc; ffloc];
      
      % Only keeping those transitions identified by both the forward and backward predictors.
      for PCT = 1:lenP
        [~, didx] = unique(ftransloc(:,PCT));
        ftransloc(didx,PCT) = 0;
      end
      
      %Making an array of zeros and ones, where ones signify transitions.
      ftransindex = zeros(lendata,lenP);
      
      for PCT = 1:lenP
        for idx = 1:length(ftransloc)
          if ftransloc(idx,PCT) ~= 0
            ftransindex(ftransloc(idx,PCT),PCT) = 1;
          end
        end
      end
      
      CKtransitions = sum(ftransindex,2);
      
      % Applying a t-test to the FRET states between each transition, and only keeping significantly different states.
      alltransindex = find(CKtransitions(:,1));
      boundaries = [0; alltransindex; lendata];
      if max(boundaries) ~= boundaries(end)
        boundaries(boundaries==max(boundaries))=[];
      end
      
      if ttestswitch == 1
        ttestctr = 1;
        while ttestctr < (length(boundaries)-2)
          testregion = boundaries(ttestctr)+separation:boundaries(ttestctr+1)-separation;
          testnextregion = boundaries(ttestctr+1)+separation:boundaries(ttestctr+2)-separation;
          ttestresult = ttest2(FRETinten(testregion), FRETinten(testnextregion), 'Vartype','unequal');
          if ttestresult == 1
            ttestctr = ttestctr+1;
          else
            boundaries(ttestctr+1) = [];
          end
        end
      end
      
      CKtransindex(:,1) = boundaries(2:end-1);
      baftransindex = (1:1:length(FRETinten));
      for a = 1:length(CKtransindex)
        baftransindex(CKtransindex(a))=0;
      end
      baftransindex(baftransindex==0)=[];
      for b = 1:length(baftransindex)
        CKtransitions(baftransindex(b))=0;
      end
      
    end
    %%%%%% end of Pengning's CK edge detection code %%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function addCKEdgesToCurrentPlot(axProps, oldCK)
      arguments
        axProps
        oldCK = [];
      end
      fretData = axProps.handleToMol.currentData.fret;
      
      [~, CKtransindex] = CKedgedetection(axProps, fretData, axProps.ckValues{1}, axProps.ckValues{2}, ...
        axProps.ckValues{3}, axProps.ckValues{4}, axProps.ckValues{5});
%                 [~, CKtransindex] = CKedgedetection(axProps, fretData, app.WindowSizeSpinner.Value, app.MinpeakheightSpinner.Value, ...
%         app.PercentileEditField.Value, app.SeparationSpinner.Value, app.ttestfilterCheckBox.Value);
%       if ~isempty(oldCK)
%         for i = 1:length(oldCK)
          lineToClear = findobj(axProps.ax2, 'Type', 'constantline', 'Tag', 'ck');
          delete(lineToClear)
          lineToClear = findobj(axProps.ax1, 'Type', 'constantline', 'Tag', 'ck');
          delete(lineToClear)
%         end
%       end
      for i = 1:length(CKtransindex)
        transitionSpot = CKtransindex(i);
        xline(axProps.ax2, transitionSpot, 'LineWidth', 1, 'Tag', 'ck');
        xline(axProps.ax1, transitionSpot, 'LineWidth', 1, 'Tag', 'ck');
      end
      
    end
    function createTiledLayoutForPlot(axProps)
      delete(axProps.ax1);
      delete(axProps.ax2);
      delete(axProps.ax3);
      switch axProps.currentTab
        case 'filterTab'
          switch axProps.filterInputs.thirdAxForPlot
            case true
              axProps.numAxes = 3;
            otherwise
              axProps.numAxes = 2;
          end
        otherwise
          switch axProps.smoothKmeans.CheckBox
            case true
              axProps.numAxes = 3;
            case false
              axProps.numAxes = 2;
          end
      end
      switch axProps.numAxes
        case 2
          tileSpan = 3;
          columnSpan = 3;
        case 3
          tileSpan = 2;
          columnSpan = 3;
      end
      for i = 1:axProps.numAxes
        axName = ['ax', num2str(i)];
        axProps.(axName) = nexttile(axProps.layout, [tileSpan,columnSpan]);
        set(axProps.(axName), 'Interactions', []);
      end
      axProps.plot;
    end
    %%%%%%%%% functions for setting up plot info depending on current tab
    function setupForPlottingKmeansTab(axProps)
      %reset data
      axProps.data = [];
      % top plot settings:
      axProps.titles.ax1 = "Acceptor (red) and Donor (green) Signals";
      axProps.ylabels.ax1 = "Intensity";
      axProps.data(1).ax1 = axProps.handleToMol.currentData.donor;
      axProps.data(2).ax1 = axProps.handleToMol.currentData.acceptor;
      axProps.numDatasets.ax1 = 2;
      axProps.colors.ax1 = {'g', 'r'};
      axProps.plotType.ax1 = {'plot', 'plot'};
      axProps.linewidth.ax1 = {0.5, 0.5};
      axProps.lineStyle.ax1 = {'-', '-'};
      
      % third optional plot settings - if user chose to smooth the data
      % before running kmeans on it, the smoothed trace will be the third
      % plot
      switch axProps.smoothKmeans.CheckBox
        case true
          axProps.numAxes = 3;
          axProps.plotType.ax3 = {'plot'};
          axProps.numDatasets.ax3 = 1;
          axProps.XLim.ax3.kmeansTab.limits = axProps.XLim.ax2.kmeansTab.limits;
          axProps.YLim.ax3.kmeansTab.limits = axProps.YLim.ax1.kmeansTab.limits;
          switch axProps.smoothKmeans.dataToCluster
            case {'Donor & Acceptor'}
              dataToSmooth = [axProps.handleToMol.currentData.donor, axProps.handleToMol.currentData.acceptor];
              axProps.numDatasets.ax3 = 2;
              axProps.colors.ax3 = {'g', 'r'};
              axProps.plotType.ax3 = {'plot', 'plot'};
              axProps.linewidth.ax3 = {0.5, 0.5};
              axProps.lineStyle.ax3 = {'-', '-'};
            case 'Donor'
              dataToSmooth = axProps.handleToMol.currentData.donor;
              axProps.ylabels.ax3 = "Avg Intensity";
              axProps.colors.ax3 = {'g'};
              axProps.linewidth.ax3 = {0.5};
              axProps.lineStyle.ax3 = {'-'};
            case 'Acceptor'
              dataToSmooth = axProps.handleToMol.currentData.acceptor;
              axProps.ylabels.ax3 = "Avg Intensity";
              axProps.colors.ax3 = {'r'};
              axProps.linewidth.ax3 = {0.5};
              axProps.lineStyle.ax3 = {'-'};
            case 'Fret'
              dataToSmooth = axProps.handleToMol.currentData.fret;
              axProps.ylabels.ax3 = "FRET Efficiency Smoothed";
              axProps.colors.ax3 = {'m'};
              axProps.linewidth.ax3 = {0.5};
              axProps.lineStyle.ax3 = {'-'};
              axProps.YLim.ax3.kmeansTab.limits = axProps.YLim.ax2.kmeansTab.limits;
          end
          switch axProps.smoothKmeans.smoothType
            case 'mean'
              smoothedData = movmean(dataToSmooth, axProps.smoothKmeans.DataWindowSize, 'omitnan');
            case 'median'
              smoothedData = movmedian(dataToSmooth, axProps.smoothKmeans.DataWindowSize, 'omitnan');
          end
          for i = 1:axProps.numDatasets.ax3
            axProps.data(i).ax3 = smoothedData(:,i);
          end
          axProps.titles.ax3 = "Smoothed " + convertCharsToStrings(axProps.smoothKmeans.dataToCluster);
      end % end of settings for third plot of smoothed data

      % bottom plot settings
      axProps.data(1).ax2 = axProps.handleToMol.currentData.fret;
      switch axProps.currentAnalysisStep
        case {'postClustering', 'firstAndLastRemoved', 'singleFramesAnalyzed'}
          axProps.data(2).ax2 = axProps.handleToMol.currentData.clusteredSignal;
          axProps.numDatasets.ax2 = 2;
          axProps.titles.ax2 = "FRET efficiency (pink), ClusteredSignal (blue)";
          axProps.ylabels.ax2 = "FRET & ClusterResults";
          axProps.colors.ax2 = {'m', 'b'};
          axProps.plotType.ax2 = {'plot', 'plot'};
          axProps.linewidth.ax2 = {0.5, 0.5};
          axProps.lineStyle.ax2 = {'-', axProps.lineStyleForClusteredSignalPlots};
          switch axProps.smoothKmeans.plotClusteredSignalTF
            case true
              axProps.numDatasets.ax3 = axProps.numDatasets.ax3 + 1;
              axProps.data(axProps.numDatasets.ax3).ax3 = axProps.handleToMol.currentData.clusteredSignal;
              axProps.colors.ax3{axProps.numDatasets.ax3} = 'b';
              axProps.plotType.ax3{axProps.numDatasets.ax3} = 'plot';
              axProps.linewidth.ax3{axProps.numDatasets.ax3} = 0.5;
              axProps.lineStyle.ax3{axProps.numDatasets.ax3} = axProps.lineStyleForClusteredSignalPlots;
          end
        otherwise
          axProps.numDatasets.ax2 = 1;
          axProps.titles.ax2 = "FRET efficiency";
          axProps.ylabels.ax2 = "FRET";
          axProps.colors.ax2 = {'m'};
          axProps.plotType.ax2 = {'plot'};
          axProps.linewidth.ax2 = {0.5};
          axProps.lineStyle.ax2 = {'-'};
      end
    end
    function setupForPlottingDataFilteringTab(axProps)
      %reset data
      axProps.data = [];
      % gather filter information for display
      filter = dataFilter(axProps.handleToMol.currentData, axProps.filterInputs);
      % set correct number of axes
      if axProps.filterInputs.thirdAxForPlot == true && ismember(axProps.currentTab, {'filterTab'})
        if axProps.numAxes ~= 3
          axProps.numAxes = 3;
          tileNum = 1;
          tileSpan = 2;
          columnSpan = 3;
          for i = 1:2
            axName = ['ax', num2str(i)];
            axProps.(axName).Layout.Tile = tileNum;
            axProps.(axName).Layout.TileSpan = [tileSpan,columnSpan];
            tileNum = tileNum + tileSpan*columnSpan;
          end
          axProps.ax3 = nexttile(axProps.layout, [tileSpan,columnSpan]);
          axProps.ax3.Layout.Tile = tileNum;
          set(axProps.ax3, 'Interactions', []);
        end
      else
        if axProps.numAxes ~= 2
          axProps.numAxes = 2;
          delete(axProps.ax3);
          tileSpan = 3;
          columnSpan = 3;
          tileNum = 1;
          for i = 1:axProps.numAxes
            axName = ['ax', num2str(i)];
            axProps.(axName).Layout.Tile = tileNum;
            axProps.(axName).Layout.TileSpan = [tileSpan,columnSpan];
            tileNum = tileNum + tileSpan*columnSpan;
          end
        end
      end
      % set up the base data for ax1, ax2, and ax3
      % base data is .......*********add more to comment here
      axProps.setUpBaseDataForFilterTab(filter);
      if axProps.filterInputs.threshInfo.TF == true
        axProps.getThresholdPlotData;
      end

      

%       replacementDataForAx3 = [];
%       % if the action for bleach removal involves replacement, also plot replaced
%       % values in both ax1 and ax2 in a different color than values that
%       % would just be removed.
%       
%       if ismember(axProps.filterInputs.actionToTake.action, {'Replace'})
%         %locate the rows that would be replaced with new values and
%         %remove those data points from the first and second data sets
%         %in ax1 (donor and acceptor), and from the first data set in
%         %ax2 (fret).
%         rows = axProps.locateRowsThatWereReplacedWithNewValuesInBleachSettings(filter.filteredData.donor, filter.filterIndex);
%         ax = {'ax1', 'ax1', 'ax2'};
%         dataNumbers = [1, 2, 1];
%         for i = 1:3
%           axProps.data(dataNumbers(i)).(ax{i})(rows) = nan;
%         end
%         
%         % The commented out lines below would make it so that the user
%         % will only be shown the final replacement values, not the values
%         % before replacement. It's easier to see this way but gives you
%         % less information.
%         %             axProps.data(3).ax1(rws) = nan; %only show parts that were replaced
%         %             axProps.data(4).ax1(rws) = nan;
%         %             axProps.data(2).ax2(rws) = nan;
%         
%         %Add new data sets to the ends of the data blocks for each axis
%         %data sets contain only the replacement values (all else is nan)
%         fieldNamesForData = {'donor', 'acceptor', 'fret'};
%         replacementDataForAx3 = zeros(3,length(axProps.handleToMol.currentData.fret));
%         for i = 1:3
%           x = filter.replacementValues.(fieldNamesForData{i});
%           axProps.numDatasets.(ax{i}) = axProps.numDatasets.(ax{i}) + 1;
%           axProps.data(axProps.numDatasets.(ax{i})).(ax{i}) = x;
%           replacementDataForAx3(i,:) = x;
%           axProps.colors.(ax{i}){end+1} = axProps.darkGreenRedFusha{i};
%           axProps.plotType.(ax{i}){end+1} = 'plot';
%           axProps.linewidth.(ax{i}){end+1} = 1.5;
%           axProps.lineStyle.(ax{i}){end+1} = '-';
%         end
%       end
      
     

      




      % update text info area to show info about the current molecule
     
      if axProps.updateTextAreaFlag == true
      updateTextAreaForDataFilterTab(axProps, ss, bleachData.dataToUse);
      end
    end
    function setUpBaseDataForFilterTab(axProps, filter)
      % top plot is donor and acceptor intensities
      axProps.data(1).ax1 = axProps.handleToMol.currentData.donor;
      axProps.data(2).ax1 = axProps.handleToMol.currentData.acceptor;
      % the third and fourth data sets for ax1 are the sections that would
      % be removed if current bleach settings were applied
      axProps.data(3).ax1 = axProps.data(1).ax1;
      axProps.data(4).ax1 = axProps.data(2).ax1;
      axProps.data(3).ax1(~filter.filterIndex) = nan; %only show parts that will be taken out
      axProps.data(4).ax1(~filter.filterIndex) = nan;
      axProps.numDatasets.ax1 = 4;

      axProps.titles.ax1 = "Acceptor (red) and Donor (green) Signals. Removed Sections in light blue. Replaced Sections in dark blue.";
      axProps.ylabels.ax1 = "Intensity";
      axProps.colors.ax1 = {'g', 'r', [0.6 0.8 1], [0.6 0.8 1]};
      %axProps.colors.ax1 = {[0.6 0.8 1], [0.6 0.8 1], 'g', 'r'};
      axProps.plotType.ax1 = {'plot', 'plot', 'plot', 'plot'};
      axProps.linewidth.ax1 = {0.5, 0.5, 0.5, 0.5};
      axProps.lineStyle.ax1 = {'-', '-', '-', '-'};

      % middle plot is the FRET efficiency
      axProps.data(1).ax2 = axProps.handleToMol.currentData.fret;
      axProps.data(2).ax2 = axProps.data(1).ax2;
      axProps.data(2).ax2(~filter.filterIndex) = nan;
      axProps.numDatasets.ax2 = 2;
      axProps.titles.ax2 = "FRET efficiency (pink). Removed Sections in light blue";
      axProps.ylabels.ax2 = "FRET";
      axProps.colors.ax2 = {'m', [0.6 0.8 1]};
      axProps.plotType.ax2 = {'plot', 'plot'};
      axProps.linewidth.ax2 = {0.5, 0.5};
      axProps.lineStyle.ax2 = {'-', '-'};
      % if data has already been clustered, show clustered signal too
      if isfield(axProps.handleToMol.currentData, 'clusteredSignal')
        axProps.numDatasets.ax2 = 4;
        axProps.data(3).ax2 = axProps.handleToMol.currentData.clusteredSignal;
        axProps.data(4).ax2 = axProps.handleToMol.currentData.clusteredSignal;
        axProps.data(4).ax2(~filter.filterIndex) = nan;
        axProps.colors.ax2 = {'m', [0.6 0.8 1], 'b', [0.6 0.8 1]};
        axProps.plotType.ax2 = {'plot', 'plot', 'plot', 'plot'};
        axProps.linewidth.ax2 = {0.5, 0.5, 0.5, 0.5};
        axProps.lineStyle.ax2 = {'-', '-', axProps.lineStyleForClusteredSignalPlots, '-'};
      end
      
      % add replacement values if needed
      switch axProps.filterInputs.actionToTake.action
        case 'Replace'
          axs = {'ax1', 'ax1', 'ax2'};
          dataNames = {'donor', 'acceptor', 'fret'};
          originalDataIdx = [1,2,1];
          filterDataIdx = [3,4,2];
          for i = 1:3
            replaceIdx = ~isnan(filter.replacementValues.(dataNames{i}));
            %put the original values in with the data to be filtered out
            axProps.data(filterDataIdx(i)).(axs{i})(replaceIdx) = axProps.data(originalDataIdx(i)).(axs{i})(replaceIdx);
            %replace the "original values" data with the replacements
            axProps.data(originalDataIdx(i)).(axs{i})(replaceIdx) = filter.replacementValues.(dataNames{i})(replaceIdx);
            % plot the new replacement values in a slightly different color
            % again for better viewing
            axProps.numDatasets.(axs{i}) = axProps.numDatasets.(axs{i}) + 1;
            axProps.data(axProps.numDatasets.(axs{i})).(axs{i}) = filter.replacementValues.(dataNames{i});
            axProps.colors.(axs{i}){end+1} = axProps.darkGreenRedFusha{i};
            axProps.plotType.(axs{i}){end+1} = 'plot';
            axProps.linewidth.(axs{i}){end+1} = 2;
            axProps.lineStyle.(axs{i}){end+1} = '-.';
          end
      end
      
      
      % the third plot applies if the data was smoothed or if the data used
      % was the sum of acceptor and donor
      if axProps.numAxes == 3
        if axProps.filterInputs.smoothData.TF == true
          numSmoothedDataSets = size(filter.smoothedSum,2);
          switch numSmoothedDataSets
            case 1
              axProps.data(1).ax3 = filter.smoothedSum;
              axProps.data(2).ax3 = axProps.data(1).ax3;
              axProps.data(2).ax3(~filter.filterIndex) = nan;
              axProps.numDatasets.ax3 = 2;
              axProps.plotType.ax3 = {'plot', 'plot'};%, 'yline'};
              axProps.linewidth.ax3 = {0.5, 0.5};%, 1};
              axProps.lineStyle.ax3 = {'-', '-'};%, '-'};
            case 2
              axProps.data(1).ax3 = filter.smoothedSum(:,1);
              axProps.data(2).ax3 = filter.smoothedSum(:,2);
              axProps.data(3).ax3 = axProps.data(1).ax3;
              axProps.data(4).ax3 = axProps.data(2).ax3;
              axProps.data(3).ax3(~filter.filterIndex) = nan;
              axProps.data(4).ax3(~filter.filterIndex) = nan;
              axProps.numDatasets.ax3 = 4;
              axProps.plotType.ax3 = {'plot', 'plot','plot', 'plot'};%, 'yline', 'yline'};
              axProps.linewidth.ax3 = {0.5, 0.5,0.5,0.5};%,1,1};
          end
          switch axProps.filterInputs.smoothData.smoothType
            case 'average'
              smoothName = "Average";
            case 'median'
              smoothName = "Median";
          end
                  % set colors and titles for ax3 depending on the data displayed
        switch axProps.filterInputs.dataToUse 
          case {'Donor AND Acceptor', 'Donor OR Acceptor'}
            axProps.titles.ax3 = "Sliding " + smoothName + " of Donor and Acceptor. Removed Sections in light blue";
            axProps.ylabels.ax3 = smoothName + " Intensity";
            axProps.colors.ax3 = {'r','g', [0.6 0.8 1], [0.6 0.8 1]};
            axProps.lineStyle.ax3 = {'-', '-', '-', '-'};
          case 'Sum Acceptor Donor'
            axProps.titles.ax3 = "Sliding " + smoothName + " of Sum(Donor + Acceptor). Removed Sections in light blue";
            axProps.ylabels.ax3 = smoothName + " Sum Intensity";
            axProps.colors.ax3 = {'b', [0.6 0.8 1]};
            axProps.lineStyle.ax3 = {'-', '-'};
          case {'Acceptor', 'Donor', 'Fret'}
            axProps.titles.ax3 = "Sliding " + smoothName + " of " + string(axProps.filterInputs.dataToUse) + ". Removed Sections in light blue";
            axProps.ylabels.ax3 = smoothName + " Intensity";
            axProps.lineStyle.ax3 = {'-', '-'};
            switch axProps.filterInputs.dataToUse
              case 'Acceptor'
                axProps.colors.ax3 = {'r', [0.6 0.8 1]};
              case 'Donor'
                axProps.colors.ax3 = {'g', [0.6 0.8 1]};
              case 'Fret'
                axProps.colors.ax3 = {'m', [0.6 0.8 1]};
                axProps.ylabels.ax3 = smoothName + " FRET";
            end
        end
        else %data is sum acceptor donor, not smoothed, plot that on 3rd axis
          %otherwise delete 3rd axis
          switch axProps.filterInputs.dataToUse 
            case 'Sum Acceptor Donor'
              axProps.data(1).ax3 = filter.dataToBeFiltered;
              axProps.data(2).ax3 = axProps.data(1).ax3;
              axProps.data(2).ax3(~filter.filterIndex) = nan;
              axProps.numDatasets.ax3 = 2;
              axProps.plotType.ax3 = {'plot', 'plot'};
              axProps.linewidth.ax3 = {0.5, 0.5};
              axProps.lineStyle.ax3 = {'-', '-'};
              axProps.titles.ax3 = "Sum(Donor + Acceptor). Removed Sections in light blue";
              axProps.ylabels.ax3 = "Sum Intensity";
              axProps.colors.ax3 = {'b', [0.6 0.8 1]};
            case {'Acceptor', 'Donor', 'Fret','Donor AND Acceptor', 'Donor OR Acceptor'}
              axProps.numAxes = 2;
              
              delete(axProps.ax3);
              tileSpan = 3;
              columnSpan = 3;
              for i = 1:axProps.numAxes
                axName = ['ax', num2str(i)];
                axProps.(axName) = nexttile(axProps.layout, [tileSpan,columnSpan]);
                set(axProps.(axName), 'Interactions', []);
              end
          end
        end
        if axProps.filterInputs.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
          axProps.numDatasets.ax1 = axProps.numDatasets.ax1 +1;
          axProps.data(axProps.numDatasets.ax1).ax1 = axProps.filterInputs.redFramesRequirements.redFramesThresh;
          axProps.colors.ax1{end+1} = axProps.darkGreenRedFusha{2};
          axProps.plotType.ax1{end+1} = 'yline';
          axProps.linewidth.ax1{end+1} = 0.5;
          axProps.lineStyle.ax1{end+1} = '-';
        end

      end
    end
    
    function getThresholdPlotData(axProps)
        % plot threshold lines if needed.
        % there can be up to 4 threshold inputs
        % if data was smoothed, plot threshold on ax3 and axis of data,
        % otherwise just plot on axis of data
        if axProps.filterInputs.smoothData.TF == true
          numAxesToApplyTo = 2;
        else
          numAxesToApplyTo = 1;
        end
        %bottom filters only apply when data is donor and acceptor
          switch isfield(axProps.filterInputs.threshInfo, 'bottom')
            case true
              axesToApplyTo = {'ax1', 'ax3'};
              topOrBottom = {'bottom', 'top'};
              minOrMax = {'min', 'max'};
              for i = 1:2
                for j = 1:2
                  if isfield(axProps.filterInputs.threshInfo.(topOrBottom{i}), minOrMax{j})
                    for k = 1:numAxesToApplyTo
                      axProps.numDatasets.(axesToApplyTo{k}) = axProps.numDatasets.(axesToApplyTo{k}) + 1;
                      axProps.data(axProps.numDatasets.(axesToApplyTo{k})).(axesToApplyTo{k}) = axProps.filterInputs.threshInfo.(topOrBottom{i}).(minOrMax{j});
                      axProps.colors.(axesToApplyTo{k}){end+1} = axProps.darkGreenRedFusha{i};
                      axProps.plotType.(axesToApplyTo{k}){end+1} = 'yline';
                      axProps.linewidth.(axesToApplyTo{k}){end+1} = 1;
                      axProps.lineStyle.(axesToApplyTo{k}){end+1} = '-';
                    end
                  end
                end
              end
            case false
              switch axProps.filterInputs.dataToUse
                case 'Sum Acceptor Donor'
                  axesToApplyTo = {'ax3'};
                  numAxesToApplyTo = 1;
                case {'Acceptor', 'Donor'}
                  axesToApplyTo = {'ax1', 'ax3'};
                case 'Fret'
                  axesToApplyTo = {'ax2', 'ax3'};
              end
              minOrMax = {'min', 'max'};
              for j = 1:2
                if isfield(axProps.filterInputs.threshInfo.top, minOrMax{j})
                  for k = 1:numAxesToApplyTo
                    axProps.numDatasets.(axesToApplyTo{k}) = axProps.numDatasets.(axesToApplyTo{k}) + 1;
                    axProps.data(axProps.numDatasets.(axesToApplyTo{k})).(axesToApplyTo{k}) = axProps.filterInputs.threshInfo.top.(minOrMax{j});
                    axProps.colors.(axesToApplyTo{k}){end+1} = 'b';
                    axProps.plotType.(axesToApplyTo{k}){end+1} = 'yline';
                    axProps.linewidth.(axesToApplyTo{k}){end+1} = 1;
                    axProps.lineStyle.(axesToApplyTo{k}){end+1} = '-';
                  end
                end
              end
          end
    end
    function updateTextAreaForDataFilterTab(axProps, ss, nameOfDataUsed)
      oldText = axProps.textArea.Value;
      [row, ~] = find(contains(oldText, 'Fret Centers:'), 1);
      if ~isempty(row)
        idx = row + 2; %size(oldText,1);
      else
        idx = 2;
      end
      if size(oldText,1) > 5
        oldText(idx:end) = [];
      end
      oldText{idx} = [nameOfDataUsed,' stats current molecule:'];
      oldText{idx + 1} = ['mean: ',num2str(mean(ss))];
      oldText{idx + 2} = ['std: ',num2str(std(ss))];
      oldText{idx + 3} = ['range: [',num2str(min(ss)), ' ' , num2str(max(ss)), ']'];
      oldText{idx + 4} = ['absolute median deviation: ',num2str(mad(ss,1))];
      oldText{idx + 5} = ['absolute mean deviation: ',num2str(mad(ss))];
      axProps.textArea.Value = oldText;
    end

    %%%%%%%%%%%%%%%%




    function plot(axProps)
      axProps.data = struct();
      axProps.lineStyleForClusteredSignalPlots = '-';
      switch axProps.currentTab
        case 'filterTab'
          setupForPlottingDataFilteringTab(axProps);
        otherwise
          axProps.currentTab = 'kmeansTab';
          setupForPlottingKmeansTab(axProps);
      end
        axProps.plotTrace;
    end
    function plotTrace(axProps)
      switch axProps.xUnits
        case 'Frames'
          rate = 1;
        case 'seconds'
          rate = axProps.FrameRate/1000;
        case 'miliseconds'
          rate = axProps.FrameRate;
      end
      t = 1:length(axProps.data(1).ax1);
      t = t.*rate;
      for n = 1:axProps.numAxes
        axField = ['ax', num2str(n)];
        ax = axProps.(axField);
        for i = 1:axProps.numDatasets.(axField)
          if i == 1
            ax.NextPlot = 'replacechildren';
          elseif i == 2
            ax.NextPlot = 'add';
          end
          switch axProps.plotType.(axField){i}
            case 'plot'
              plot(ax, t', axProps.data(i).(axField),'color', axProps.colors.(axField){i}, 'LineWidth', axProps.linewidth.(axField){i}, 'LineStyle', axProps.lineStyle.(axField){i});
              hold(ax, 'on');
            case 'yline'
              yline(ax, axProps.data(i).(axField),'color', axProps.colors.(axField){i}, 'LineWidth', axProps.linewidth.(axField){i}, 'LineStyle', axProps.lineStyle.(axField){i});
          end
        end
        ax.XLim = (axProps.XLim.(axField).(axProps.currentTab).limits).*rate;
        ax.YLim = axProps.YLim.(axField).(axProps.currentTab).limits;
        wholeAxisProps = {'titles', 'ylabels', 'xlabels'};
        applyProp = {@(ax,w) title(ax,w), @(ax,w) ylabel(ax,w), @(ax,w) xlabel(ax,w)};
        for w = 1:length(wholeAxisProps)
          if isfield(axProps.(wholeAxisProps{w}), axField) && ~isempty(axProps.(wholeAxisProps{w}).(axField))
            applyProp{w}(ax, axProps.(wholeAxisProps{w}).(axField))
          end
        end
        ax.XMinorTick = true;
        ax.YMinorTick = true;
        ax.XGrid = true;
        ax.YGrid = true;
        hold(ax, 'off');
      end % end of loop through axes
      % add title
      [~,name,~] = fileparts(axProps.handleToMol.fileName);
      if ~isempty(axProps.handleToMol.nameOfExperiment)
        axProps.titles.layout = [axProps.handleToMol.nameOfExperiment, '_', name];
      else
        axProps.titles.layout = [name, '_mol', num2str(axProps.handleToMol.moleculeNumber)];
      end
      title(axProps.layout, axProps.titles.layout, 'Interpreter', 'none');
      xlabel(axProps.layout, axProps.xUnits, 'Interpreter', 'none');
      if axProps.addCKEdges == true
        axProps.addCKEdgesToCurrentPlot;
      end
    end
    
    %%%%%%%%% functions that update values from listener notifications
    function selectedTabChanged(axProps, evnt)
      % called from main app's callback function whenever tab is changed
      axProps.currentTab = evnt.data{1};
      switch axProps.currentTab 
        case 'dwellTab'
          setupForPlottingDwellTimes(axProps, evnt.data{2})
        otherwise
          axProps.createTiledLayoutForPlot;
      end
      
    end
    function updateMolNum(axProps,  newMolNum, molArray)
      axProps.handleToMol = molArray(newMolNum);
      delete(axProps.listeners.analysisStep);
      axProps.listeners.analysisStep = addlistener(axProps.handleToMol,'currentAnalysisStep','PostSet',@(src,evnt) axProps.plot);
      axProps.plot;
    end
    function value = get.currentAnalysisStep(axProps)
      value = axProps.handleToMol.currentAnalysisStep;
    end
    function updateValuesForKmeans(axProps, evntData)
      axProps.smoothKmeans.(evntData.Source.Tag) = evntData.Value;
      switch evntData.Source.Tag
        case 'dataToCluster'
          switch evntData.Value
            case 'Fret'
              axProps.smoothKmeans.plotClusteredSignalTF = true;
            otherwise
              axProps.smoothKmeans.plotClusteredSignalTF = false;
          end
      end
      axProps.createTiledLayoutForPlot;
    end
    function updateBleachSettings(axProps,evntData)
      % called from main app whenever bleach settings are changed
      propName = evntData.Source.Tag;
      propValue = evntData.Value;
      axProps.bleachSettings.(propName) = propValue;
      switch axProps.currentTab
        case 'filterTab'
          axProps.plot;
      end
    end
    function updateFrameRateSettings(axProps,newValue, oldValue)
      switch class(newValue)
        case 'char'
          axProps.xUnits = newValue;
        otherwise
          axProps.FrameRate = newValue;
      end
      axProps.plot;
    end
    function updateFilterInputs(axProps,newFilterInputs)
      axProps.filterInputs = newFilterInputs;
      axProps.plot;
    end
    %%%%%%%%%%%%%%% tool bar functions
    function setUpToolbar(axProps)
      axProps.toolBar = axtoolbar(axProps.layout);
      btn = axtoolbarbtn(axProps.toolBar, 'state');
      btn.Icon = 'zoomin';
      btn.Tag = 'stateBtn';
      btn.ValueChangedFcn = @(s,d)axProps.axToolbarBtnPushed(s,d);
      btn = axtoolbarbtn(axProps.toolBar, 'state');
      btn.Icon = 'zoomout';
      btn.Tag = 'stateBtn';
      btn.ValueChangedFcn = @(s,d)axProps.axToolbarBtnPushed(s,d);
      btn = axtoolbarbtn(axProps.toolBar, 'state');
      btn.Icon = 'export';
      btn.Tag = 'stateBtn';
      btn.ValueChangedFcn = @(s,d)axProps.axToolbarBtnPushed(s,d);
      btn = axtoolbarbtn(axProps.toolBar, 'push');
      btn.Icon = 'restoreview';
      btn.ButtonPushedFcn = @(s,d)axProps.axToolbarBtnPushed(s,d);
      btn = axtoolbarbtn(axProps.toolBar, 'state');
      btn.Icon = 'pan';
      btn.Tag = 'stateBtn';
      btn.ValueChangedFcn = @(s,d)axProps.axToolbarBtnPushed(s,d);
    end
    function axToolbarBtnPushed(axProps, ~, evnt)
      %https://www.mathworks.com/help/matlab/creating_plots/control-axes-interactions.html#mw_25584f72-a016-4f23-8d7d-895394750442
      disp('tool bar btn pressed');
      disp(evnt.Source.Icon);
      switch evnt.Source.Icon
        case 'zoomin'
          zoomOutBtn = findobj(axProps.toolBar, 'Icon', 'zoomout');
          set(zoomOutBtn, 'Value', false);
          %captureImageBtn = findobj(axProps.toolBar, 'Icon', 'export');
          %set(captureImageBtn, 'Value', false);
          switch evnt.Source.Value
            case 'on'
              set(evnt.Axes, 'Interactions', [regionZoomInteraction('Dimensions', 'xy'), zoomInteraction]);
              axes = evnt.Axes;
              axProps.toolBar.UserData = listener(axes, 'XLim', 'PostSet', @(s,e) set(axes, 'XLim', e.AffectedObject.XLim));
              %axProps.toolBar.UserData = listener(axes, 'XLim', 'PostSet', @(s,e) axProps.toolBarButton_zoom(e, axes));
            case 'off'
              set(evnt.Axes, 'Interactions', []);
              delete(axProps.toolBar.UserData);
          end
        case 'zoomout'
          zoomInBtn = findobj(axProps.toolBar, 'Icon', 'zoomin');
          set(zoomInBtn, 'Value', false);
          %captureImageBtn = findobj(axProps.toolBar, 'Icon', 'export');
          %set(captureImageBtn, 'Value', false);
          switch evnt.Source.Value
            case 'on'
              set(evnt.Axes, 'Interactions', zoomInteraction);
              axes = evnt.Axes;
              axProps.toolBar.UserData = listener(axes, 'XLim', 'PostSet', @(s,e) set(axes, 'XLim', e.AffectedObject.XLim));
              %axProps.toolBar.UserData = listener(axes, 'XLim', 'PostSet', @(s,e) axProps.toolBarButton_zoom(e, axes));
            case 'off'
              set(evnt.Axes, 'Interactions', []);
              delete(axProps.toolBar.UserData);
          end
        case 'export'

          switch evnt.Source.Value
            case true
              axProps.captureImageMode = true;
              %set(evnt.Axes, 'Interactions', [regionZoomInteraction('Dimensions', 'xy'), zoomInteraction]);
              %axes = evnt.Axes;
              %axProps.toolBar.UserData = listener(axes, 'XLim', 'PostSet', @(s,e) set(axes, 'XLim', e.AffectedObject.XLim));
              
            case false
              axProps.captureImageMode = false;
              %set(evnt.Axes, 'Interactions', []);
              %delete(axProps.toolBar.UserData);
          end
        case 'restoreview'
          axProps.toolBarButton_restoreAxisLimits;
        case 'pan'
          zoomInBtn = findobj(axProps.toolBar, 'Icon', 'zoomin');
          set(zoomInBtn, 'Value', false);
          zoomInBtn = findobj(axProps.toolBar, 'Icon', 'zoomin');
          set(zoomInBtn, 'Value', false);
          set(evnt.Axes, 'Interactions', [dataTipInteraction, panInteraction]);
      end
    end
%     function toolBarButton_zoom(axProps,e, axes)
%       set(axes, 'XLim', e.AffectedObject.XLim)
%     end
    function toolBarButton_restoreAxisLimits(axProps, varargin)
      switch axProps.xUnits
        case 'Frames'
          rate = 1;
        case 'seconds'
          rate = axProps.FrameRate/1000;
        case 'miliseconds'
          rate = axProps.FrameRate;
      end
      for i = 1:axProps.numAxes
        axName = ['ax', num2str(i)];
        tabNames = {'kmeansTab', 'filterTab'};
        for j = 1:2
        axProps.YLim.(axName).(tabNames{j}).limits = axProps.YLim.(axName).(tabNames{j}).default;
        axProps.XLim.(axName).(tabNames{j}).limits = axProps.XLim.(axName).(tabNames{j}).default*rate;
        end
        axProps.(axName).XLim = axProps.XLim.(axName).(axProps.currentTab).limits;
        axProps.(axName).YLim = axProps.YLim.(axName).(axProps.currentTab).limits;
      end
    end
    %%%%%%%%%%%%%%%%%%
    
  end
  methods (Static)
    function rows = locateRowsThatWereReplacedWithNewValuesInBleachSettings(resultsFromBleachData, bleachIndex)
      resultsFromBleachData(~bleachIndex) = nan;
      [rows, ~] = find(~isnan(resultsFromBleachData));
    end
  end
  
end
