function stats = calculateClusterStats(k, clusterIndex, data)
%returns struct with each field beign a different statistic.
%each field is a k x 3 matrix with row numbers representing the 
%cluster number, and columns representing which data, ordered: fret, 
%acceptor, donor. So for example,stats.avg(2,3) is the average of the donor
%values of the 2nd cluster. stats.median(1,2) is the median of the acceptor
%values for the 1st cluster
%stats.numberOfDataPoints and stats.numberOfSections only have one column
    stats.avg = zeros(k,3);
    stats.median = zeros(k,3);
    stats.standardDeviation = zeros(k,3);
    stats.medianAbsoluteDeviation = zeros(k,3);
    stats.meanAbsoluteDeviation = zeros(k,3);
    stats.numberOfDataPoints = zeros(k,1);
    stats.numberOfSections = zeros(k,1);
  for j = 1:k
    %calculate how many segments there are in this cluster:
    index = clusterIndex(:,j);
    b = [index(2:end); nan];
    [idxRowEnd, ~] = find(b ~= index);
    stats.numberOfSections(j) = length(idxRowEnd);
    %uncomment two lines below if you want to caluclate more stats
    %about sections
    %c = [nan; index(1:end-1)];
    %[idxRowStart, ~] = find(c~= index);
    % calculate stats for each type of data in this cluster
    donorValuesClusterJ = data(clusterIndex(:,j) == 1,1);
    acceptorValuesClusterJ = data(clusterIndex(:,j) == 1,2);
    fretValuesClusterJ = data(clusterIndex(:,j) == 1,3);
    stats.numberOfDataPoints(j) = length(donorValuesClusterJ);
    stats.avg(j,:) = [mean(fretValuesClusterJ), mean(acceptorValuesClusterJ), mean(donorValuesClusterJ)];
    stats.median(j,:) = [median(fretValuesClusterJ), median(acceptorValuesClusterJ), median(donorValuesClusterJ)];
    stats.standardDeviation(j,:) = [std(fretValuesClusterJ), std(acceptorValuesClusterJ), std(donorValuesClusterJ)];
    stats.medianAbsoluteDeviation(j,:) = [mad(fretValuesClusterJ,1), mad(acceptorValuesClusterJ,1), mad(donorValuesClusterJ,1)];% Flag is set to 1 for the median absolute deviation
    stats.meanAbsoluteDeviation(j,:) = [mad(fretValuesClusterJ), mad(acceptorValuesClusterJ), mad(donorValuesClusterJ)];
  end %end of loop on j

%%% old code, delete once new code is tested and works.
% dataNames = {'donor', 'acceptor', 'fret'};
% averages = []; stats = [];
% for i = 1:3 %loop through each data type: donor, acceptor, fret.
%   allData = data(:,i);
%   for j = 1:k
%     clusterFieldname = "cluster" + num2str(j);
%     clusterJData = allData(clusterIndex(:,j) == 1);
%     averages.(
%     averages.(clusterFieldname).(dataNames{i}).avg = mean(clusterJData);
%     averages.(clusterFieldname).(dataNames{i}).med = median(clusterJData);
%     averages.(clusterFieldname).(dataNames{i}).sd = std(clusterJData);
%     averages.(clusterFieldname).numberOfDataPoints = length(clusterJData);
%     %calculate how many segments there are in this cluster:
%     index = clusterIndex(:,j);
%     b = [index(2:end); nan];
%     [idxRowEnd, ~] = find(b ~= index);
%     %uncomment two lines below if you want to caluclate more stats
%     %about sections
%     %c = [nan; index(1:end-1)];
%     %[idxRowStart, ~] = find(c~= index);
%     stats.(clusterFieldname).numberOfDataPoints = length(clusterJData);
%     stats.(clusterFieldname).numberOfSections = length(idxRowEnd);
%     
%   end %end of loop on j
% end
end
