classdef dataFilter
  properties
    inputData
    filterIndex
    filteredData

  end
  properties (Transient)
    smoothedSum
    replacementValues
    dataToBeFiltered

    
  end
%   properties (Dependent)
%     smoothedSum
%   end
  
  %%%% different data filtering options:
  %%% REMOVE:
  %%% 1. if dataUsed < threshold
  %%% 2. if dataUsed < threshold or dataUsed > maxThresh
  %%% 
  %%% REMOVE IF:
  %%% 3. if dataUsed < threshold and 'Sections Before And After Are Bleached';
  %%% 4. if dataUsed < threshold and 'Section is at begining or end'
  %%% 5. if dataUsed < threshold and 'Red Frames Are Bleached' and section
  %%% is next to red frame section
  %%% 6. if dataUsed < threshold and length(section) < minLength
  %%% 7. if dataUsed < threshold and length(section) > minLength
  %%% 
  %%%'Specify min length of sections to be removed'; 'Section is at begining or end'; ...
               % 'Red Frames Are Bleached'
               
               
  %%% Rewrite constructor function like this:
  % switch filterType, case "Threshold", case 'Whole Molecule'
  %case 'Whole Molecule'
  % check TF for conditions
  
  methods
    function filter = dataFilter(inputData, filterOptions)
      % determine data to use
      dataToFilter = filter.defineWhichDataToUse(inputData, filterOptions.dataToUse);
      filter.dataToBeFiltered = dataToFilter;
      % smooth data if selected
      if filterOptions.smoothData.TF == true
        dataToFilter = filter.smoothData(dataToFilter, filterOptions.smoothData.smoothType, filterOptions.smoothData.windowSize);
        filter.smoothedSum = dataToFilter;
      end
      % apply filter to make an index of values to set to nan
      % index is a logical TF array. True means the data points will be set
      % to nan, false means they will be left alone
      if filterOptions.threshInfo.TF == true
        index = filter.applyThresholds(dataToFilter, filterOptions.threshInfo, filterOptions.dataToUse);
      else
        index = ones(size(dataToFilter), 'logical');
      end
      % check to see if we need a red frame index
      if filterOptions.redFramesRequirements.TF == true 
        [redFrameIndex, frontBleached, backBleached] = filter.analyzeRedFrames(filterOptions.redFramesRequirements,...
          filterOptions.smoothData, inputData);
        filterOptions.redFramesRequirements.frontBleached = frontBleached;
        filterOptions.redFramesRequirements.backBleached = backBleached;
        if filterOptions.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
          if backBleached == false
            index = zeros(length(dataToFilter),1,'logical');
            filter.replacementValues = filter.calculateReplacementValues(index, inputData, filterOptions.actionToTake);
            filter.filteredData = filter.applyFilterToData(inputData, index, filter.replacementValues);
            filter.filterIndex = index;
            return
          end
%           if backBleached == true && ismember(filterOptions.filterType, 'Whole Molecule')
%             index = ones(length(dataToFilter),1,'logical');
%             dataNames = fieldnames(inputData);
%             for i = 1:3
%               filter.filteredData.(dataNames{i}) = nan(size(index));
%             end
%             filter.filterIndex = index;
%             return
%           end
        elseif filterOptions.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
          if backBleached == true
            index = zeros(length(dataToFilter),1,'logical');
            filter.replacementValues = filter.calculateReplacementValues(index, inputData, filterOptions.actionToTake);
            filter.filteredData = filter.applyFilterToData(inputData, index, filter.replacementValues);
            filter.filterIndex = index;
            return
          end
%           if backBleached == false && ismember(filterOptions.filterType, 'Whole Molecule')
%             index = ones(length(dataToFilter),1,'logical');
%             dataNames = fieldnames(inputData);
%             for i = 1:3
%               filter.filteredData.(dataNames{i}) = nan(size(index));
%             end
%             filter.filterIndex = index;
%             return
%           end
        end
      else
        redFrameIndex = zeros(size(dataToFilter(:,1)), 'logical');
      end

      %still need to address: only apply if red frames are bleached
      %checkbox & ignore red frame sections checkbox

      % adjust index to meed additional conditions
      if filterOptions.addConditions.TF == true
        % apply conditions to index depending on what the conditions are
        if contains(filterOptions.addConditions.condition, 'Specify')
          index = filter.filterSpecifiedLengths(index, filterOptions.addConditions.minOrMax, filterOptions.addConditions.lengthRequirement);
        else
          switch filterOptions.addConditions.condition
            case 'Sections Before and After are Bleached'
              index = filter.onlyApplyIfSectionsBeforeAndAfterAreBleached(index, dataToFilter,filterOptions.addConditions, redFrameIndex);
            case 'Section is at Begining or End'
              % only want to filter sections that meet threshold requirements
              % and are at the begining or end of the molecule trace
              index = filter.onlyApplyFilterToSectionsAtBeginingOrEnd(index, dataToFilter,filterOptions.addConditions, filterOptions.redFramesRequirements, redFrameIndex);
          end
        end
      end
      %add padding if option was selected
      if filterOptions.addPadding.TF == true
        index = filter.padEndsOfBleachedSections(index, filterOptions.addPadding.amount);
      end
      
      %calculates replacement values if needed or retruns arrays of nan
      %values if action is to remove
       filter.replacementValues = filter.calculateReplacementValues(index, inputData, filterOptions.actionToTake);
      
      % finally, apply filter to data. filteredData = struct with fields,
      % donor, acceptor, and fret
      filter.filteredData = filter.applyFilterToData(inputData, index, filter.replacementValues);
      filter.filterIndex = index;
    end
  end
    methods 
    function dataToFilter = defineWhichDataToUse(~,data, dataDescription)
      % define which data to use based on drop down item selected by user
      switch dataDescription
        case 'Sum Acceptor Donor'
          dataToFilter(:,1) = data.donor + data.acceptor;
        case {'Acceptor', 'Donor', 'Fret'}
          dataName = lower(dataDescription);
          dataToFilter(:,1) = data.(dataName);
        case {'Donor AND Acceptor', 'Donor OR Acceptor'}
          dataToFilter(:,1) = data.donor;
          dataToFilter(:,2) = data.acceptor;
        case 'all' %this is when an option was selected to remove the whole trace
          dataToFilter(:,1) = nan(size(data.acceptor));
      end
    end
    function dataSmoothed = smoothData(~,data, smoothType, windowSize)
      % smooth data based on user selections 
      nanIdx = isnan(data); %make sure nan index is not changed
      % smooth the data
      switch smoothType %user selected drop down item.
        case 'average'
          smoothFcn = @(data, windowSize) movmean(data, windowSize, 'omitnan'); %'omitnan' flag tells function to ignore nan values in calculation...
          %Results in mean calculated over fewer points if nan values are in window
        case 'median'
          smoothFcn = @(data, windowSize) movmedian(data, windowSize, 'omitnan');
      end
      dataSmoothed = data;
      for i = 1:size(data, 2) %size is 1 unless dataToUse is 'Donor OR Acceptor' or 'Donor AND Acceptor'
        dataSmoothed(:,i) = smoothFcn(data(:,i), windowSize);
        dataSmoothed(nanIdx, i) = nan;
      end
    end
    function finalIndex = applyThresholds(~,data, threshInfo, dataDescription)
      % index returned is a TF array where true == data to be filtered out,
      % and false = data does stays the same.
      index = zeros(length(data),4,'logical');
      % threshInfo struct has two fields: 'top' and 'bottom'
      % bottom will only apply if there are two data sets defined by 'Data
      % To Use' drop down in UI
      % Each threshInfo.(top or bottom) field is a structure
      % with fields: 'max' and/or 'min' depending on user selections
      % Go through all possible filters an apply the ones that are defined
      % by the threshInfo struct
      if isfield(threshInfo.top, 'min')
        index(:,1) = data(:,1) < threshInfo.top.min;
      end
      if isfield(threshInfo.top, 'max')
        index(:,2) = data(:,1) > threshInfo.top.max;
      end
      threshIndex = zeros(size(data), 'logical');
      threshIndex(:,1) = or(index(:,1), index(:,2));
      if size(data,2) == 2
        if isfield(threshInfo.bottom, 'min')
          index(:,3) = data(:,2) < threshInfo.bottom.min;
        end
        if isfield(threshInfo.bottom, 'max')
          index(:,4) = data(:,2) > threshInfo.bottom.max;
        end
        threshIndex(:,2) = or(index(:,3), index(:,4));

      end

      switch dataDescription
        case 'Donor AND Acceptor'
          finalIndex = and(threshIndex(:,2), threshIndex(:,1));
        case 'Donor OR Acceptor'
          finalIndex = or(threshIndex(:,2), threshIndex(:,1));
        otherwise
          finalIndex = threshIndex(:,1);
      end
      %note: values in data that are nan will automatically be returned as
      %false - this is what we want
    end
    function [redFrameIndex, frontBleached, backBleached] = analyzeRedFrames(filter,redFramesRequirements, smoothData, inputData)
      switch smoothData.TF
        case true
          acceptorData = filter.smoothData(inputData.acceptor, smoothData.smoothType, smoothData.windowSize);
        case false
          acceptorData = inputData.acceptor;
      end
      greenStart = redFramesRequirements.greenStartFrame;
      greenEnd = redFramesRequirements.greenEndFrame;
      if redFramesRequirements.onlyApplyIfRedFramesAreBleached == true || redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
        % laser switch time only matters if we need to know if red frames
        % are bleached. If we only want to know red frame index, then don't
        % take out laser switch frames
        if isfield(redFramesRequirements, 'laserSwitch')
          greenStart = redFramesRequirements.greenStartFrame - redFramesRequirements.laserSwitch;
          greenEnd = redFramesRequirements.greenEndFrame + redFramesRequirements.laserSwitch;
        end
      end
      redFrameIndex = zeros(length(acceptorData),1,'logical');
      redFrameIndex(1:greenStart) = true;
      if greenEnd ~= inf && greenEnd <= length(acceptorData)
        redFrameIndex(greenEnd:end) = true;
      end
      if redFramesRequirements.onlyApplyIfRedFramesAreBleached == true || redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
        [frontBleached, backBleached] = filter.determineIfRedFramesAreBleached(greenStart, greenEnd,  ...
          acceptorData, redFramesRequirements.redFramesThresh);
      else
        frontBleached = []; backBleached = [];
      end
    end
    function index = filterSpecifiedLengths(filter,index, minOrMax, lengthRequirement)
      % using filter index made from applying thresholds, separate the
      % parts to be filtered out into sections and make sure they meet the
      % length requirements
      switch minOrMax
        case 'min'
          meetsLengthSpecifications = @(sectionIndex) length(sectionIndex) >= lengthRequirement;
        case 'max'
          meetsLengthSpecifications = @(sectionIndex) length(sectionIndex) <= lengthRequirement;
      end
      [idxRowStart, idxRowEnd] = filter.separateIndexIntoSections(index, 'filteredSectionsOnly');
      for i = 1:length(idxRowStart)
        sectionIndex = idxRowStart(i):idxRowEnd(i);
        if meetsLengthSpecifications(sectionIndex) == false
          index(sectionIndex) = false;
        end
      end
    end
    function newIndex = onlyApplyFilterToSectionsAtBeginingOrEnd(filter,index, data, conditions, redFramesRequirements, redFrameIndex)
      % modify the index of data to filter so that only the first and last
      % sections keep the filter. The rest of the sections stay as they are
      arguments
        filter
        index
        data
        conditions
        redFramesRequirements
        redFrameIndex = zeros(size(index), 'logical')
      end
      % convert input index from logical to numeric
      index = double(index);
      % set already defined nan sections to 1 and red Frame index to 4. 
      index(isnan(data(:,1))) = 1;
      % set red frame index to 4. Doing this after setting nan index to 3
      % makes it so that even if red frames were already removed, they will
      % still show up as 4
      if conditions.excludeRedFrames == true
      index(redFrameIndex) = 4;
      end
      % separate index into sections
      [idxRowStart, idxRowEnd] = filter.separateIndexIntoSections(index);
      % make a new index that is false for all data points. This will be
      % the output index that is false for all except the begining and end
      % sections
      newIndex = zeros(size(index), 'logical');
      %if only apply if red frames are bleached option is selected,
      if redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
        if redFramesRequirements.frontBleached == false && redFramesRequirements.backBleached == false
          return
        end
      elseif redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
        if redFramesRequirements.frontBleached == true && redFramesRequirements.backBleached == true
          return
        end
      end

      % look at first section. If it is the red frame section, then the
      % second section is the "first section" to look at
      switch index(idxRowStart(1))
        case 1 %(true)
          % if the first section is true in the old index, then make it
          % true in the new index
          newIndex(idxRowStart(2):idxRowEnd(2)) = true;
        case 4
          % look at second section if the number of sections is > 1
          % if second section is true (aka. 1) then set it as true in the new
          % index. Otherwise keep the begining of the new index set to false
          if length(idxRowStart) > 1 && index(idxRowStart(2)) == 1
%             if redFramesRequirements.frontBleached == true && redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
              newIndex(idxRowStart(2):idxRowEnd(2)) = true;
%             end
%             if redFramesRequirements.frontBleached == false && redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
%               newIndex(idxRowStart(2):idxRowEnd(2)) = true;
%             end
          end
        otherwise % do nothing
      end
      %now do the same with the last section.
      switch index(idxRowStart(end))
        case 1
          newIndex(idxRowStart(end):idxRowEnd(end)) = true;
        case 4
          if length(idxRowStart) > 1 && index(idxRowStart(end-1)) == 1
%             if redFramesRequirements.backBleached  == true && redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
              newIndex(idxRowStart(end-1):idxRowEnd(end-1)) = true;
%             end
%             if redFramesRequirements.backBleached  == false && redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
%               newIndex(idxRowStart(2):idxRowEnd(2)) = true;
%             end
          end
      end
    end
    function index = onlyApplyIfSectionsBeforeAndAfterAreBleached(filter,index, data, conditions, redFrameIndex)
      arguments
        filter
        index
        data
        conditions
        redFrameIndex = zeros(size(index), 'logical')
      end
      tempIndex = double(index);
      tempIndex(isnan(data(:,1))) = 1;
      if conditions.excludeRedFrames == true
      tempIndex(redFrameIndex) = 4;
      end
      [idxRowStart, idxRowEnd] = filter.separateIndexIntoSections(tempIndex);
      for i = 1:size(idxRowStart,1)
        %test to see if section is red frames or nan
        switch tempIndex(idxRowStart(i))
          case 0 %section will not be filtered & is not red frames or nan
            if idxRowStart(i) ~= 1 && idxRowEnd(i) ~= length(index)
              valueOfSectionBefore = tempIndex(idxRowStart(i)-1);
              valueOfSectionAfter = tempIndex(idxRowStart(i+1));
              if valueOfSectionBefore == 1 && valueOfSectionAfter == 1
                index(idxRowStart(i):idxRowEnd(i)) = false;
              else
                index(idxRowStart(i):idxRowEnd(i)) = true;
              end
            end
        end
      end
    end
    function [TFfront, TFback] = determineIfRedFramesAreBleached(~,greenStart, greenEnd,  acceptorData, thresh)
            % use acceptor data and threshold for red frames bleaching to
            % determine if the red frames bleached or not
            tempIx = acceptorData <= thresh;
            redFramesInFront = tempIx(1:greenStart);
            TFfront = all(redFramesInFront);
            if length(acceptorData) >= greenEnd && greenEnd ~= inf
              redFramesInBack = tempIx(greenEnd:end);
              TFback = all(redFramesInBack);
            else
              TFback = false;
            end
    end
    function replacementValues = calculateReplacementValues(filter,index, data, actionToTake)
      %calculate replacement values using the values immediately before and
      %after the section that needs replacement
      [idxRowStart, idxRowEnd] = filter.separateIndexIntoSections(index);
      dataFields = {'donor', 'acceptor', 'fret'};
      for j = 1:3
        replacementValues.(dataFields{j}) = nan(size(data.(dataFields{j})));
      end
      switch actionToTake.action
        case 'Replace'
%           if ~isfield(actionToTake, 'value')
%             actionToTake.value = nan;
%             actionToTake.data = 'all';
%           end
          sampleIntensityData = data.donor;
          for i = 1:length(idxRowStart)
            sectionsToUse = [];
            if index(idxRowStart(i)) == true
              sectionRows = idxRowStart(i):idxRowEnd(i);
              switch actionToTake.replace.replacementValueSpecified
                case true
                  replacementValues.donor(sectionRows) = actionToTake.replace.donor;
                  replacementValues.acceptor(sectionRows) = actionToTake.replace.acceptor;
                  fret = actionToTake.replace.acceptor/(actionToTake.replace.acceptor+actionToTake.replace.donor);
                  replacementValues.fret(sectionRows) = fret;
                case false
                  if i~= 1 && ~isnan(sampleIntensityData(idxRowStart(i)-1))
                    sectionBefore = idxRowStart(i)-1;
                    sectionsToUse = [sectionsToUse, sectionBefore]; %#ok
                  end
                  if i ~= length(idxRowStart) && ~isnan(sampleIntensityData(idxRowEnd(i)+1))
                    sectionAfter = idxRowEnd(i)+1;
                    sectionsToUse = [sectionsToUse, sectionAfter]; %#ok
                  end
                  if ~isempty(sectionsToUse)
                    for j = 1:3
                      replacementValues.(dataFields{j})(sectionRows) = mean(data.(dataFields{j})(sectionsToUse));
                    end
                  else
                    for j = 1:3
                      replacementValues.(dataFields{j})(sectionRows) = nan;
                    end
                  end
              end
            end
          end
      end
    end
    function index = padEndsOfBleachedSections(filter,index, paddingAmount)
      totalLength = length(index);
      [idxRowStart, idxRowEnd] = filter.separateIndexIntoSections(index, 'filteredSectionsOnly');
      for i = 1:length(idxRowStart)
        frontRowsToPad = (idxRowStart(i) - paddingAmount):idxRowStart(i);
        frontRowsToPad = frontRowsToPad(frontRowsToPad >= 1);
        index(frontRowsToPad) = true;
        backRowsToPad = idxRowEnd(i):(idxRowEnd(i) + paddingAmount);
        backRowsToPad = backRowsToPad(backRowsToPad <= totalLength);
        index(backRowsToPad) = true;
      end
    end
    function dataAfterFiltering = applyFilterToData(~,inputData, index, replacementValues)
      dataFields = {'donor', 'acceptor', 'fret'};
      for i = 1:3
        dataAfterFiltering.(dataFields{i}) = inputData.(dataFields{i});
        dataAfterFiltering.(dataFields{i})(index) = replacementValues.(dataFields{i})(index);
      end
    end
    end
    methods (Static)
    function [idxRowStart, idxRowEnd] = separateIndexIntoSections(index, whichSections)
      arguments
        index
        whichSections char = 'all' 
      end
      index = double(index);
      b = [index(2:end); nan];
      [idxRowEnd, ~] = find(b ~= index);
      c = [nan; index(1:end-1)];
      [idxRowStart, ~] = find(c~= index);
      switch whichSections
        case 'filteredSectionsOnly'
          index = logical(index);
          idxRowStart = idxRowStart(index(idxRowStart) == true);
          idxRowEnd = idxRowEnd(index(idxRowEnd) == true);
        case 'nonFiltered'
          index = logical(index);
          idxRowStart = idxRowStart(index(idxRowStart) == false);
          idxRowEnd = idxRowEnd(index(idxRowEnd) == false);
      end
    end
  end

end