classdef (ConstructOnLoad) dataFilterInputs 
  %this class keeps track of the information to feed to the data filter
  properties
    filterType
    dataToUse 
    smoothData %.TF, .smoothType, .windowSize
    redFramesRequirements %.TF, .greenStartFrame, .greenEndFrame, .laserSwitch, .onlyApplyIfRedFramesAreBleached, .redFramesThresh
    threshInfo; %.TF, .top.min, .top.max, .bottom.min, .bottom.max
    addConditions % .TF, .condition, .minOrMax, .lengthRequirement, .excludeRedFrames
    addPadding %.TF, .amount
    actionToTake %.action, .replace.replacementValueSpecified, .replace.donor, .replace.acceptor
    

  end
  properties
    thirdAxForPlot = false;
  end

  
  methods
    function input = dataFilterInputs(app)
      input = input.setUpDataStructure;
      fiterObjectGrids = findobj(app.CreateDataFilterPanel, 'Type', 'uigrid');
      filterObjects = [app.FilterTypeDropDown; app.DatatouseDropDown; app.GreenStartFrameEditField; ...
        app.GreenEndFrameEditField; app.numframesshutterswitchSpinner; fiterObjectGrids];
      %%% filterObjects = filterType drop down, datatoUse drop down,
      %%% uigrids of: conditions, actionToTake, redFrames, smoothdata,
      %%% padding
      %%% add numFramesShutterSwitch to red frames
      for i = 1:length(filterObjects)
        switch filterObjects(i).Tag
          case 'filterType'
            input = collectFilterTypeInfo(input, filterObjects(i).Value);
          case 'dataToUse'
            input = collectDataToUseInfo(input, filterObjects(i));
          case 'actionToTake' %.action, .replace.replacementValueSpecified, replace.donor, .replace.acceptor
            input = collectActionObjectData(input,filterObjects(i));
          case 'conditions'
            input = collectConditionObjectData(input,filterObjects(i));
          case 'redFrames'
            input = collectRedFrameObjectData(input,filterObjects(i));
          case 'smoothData'
            input = collectSmoothObjectData(input,filterObjects(i));
          case 'padding'
            input = collectPaddingObjectData(input,filterObjects(i));
          case 'thresholds'
            input = collectThresholdObjectData(input,filterObjects(i));
        end
      end
      input = determineIfRedFramesAreRequired(input);
      input = gatherPlotInfo(input);
      
    end
%     function filter = createDataFilter(filterInputs, dataToFilter)
%       filter = dataFilter(dataToFilter, filterInputs);
%     end

 
    function input = setUpDataStructure(input)
      input.smoothData = struct('TF', false, 'smoothType', char.empty, 'windowSize', []);
      input.redFramesRequirements = struct('TF', false, 'greenStartFrame', [], 'greenEndFrame', [], 'laserSwitch', [], ...
        'onlyApplyIfRedFramesAreBleached', false,'onlyApplyIfRedFramesAreNOTBleached', false, 'redFramesThresh', []);
      input.threshInfo.TF = false;
      %input.threshInfo.top = struct('min', [], 'max', []);
      %input.threshInfo.bottom = struct('min', [], 'max', []);
      input.addConditions = struct('TF', false, 'condition', char.empty, 'minOrMax', char.empty, 'lengthRequirement', [], 'excludeRedFrames', false);
      input.addPadding = struct('TF', false, 'amount', []);
      replaceStruct = struct('replacementValueSpecified', false, 'donor', [], 'acceptor', []);
      input.actionToTake = struct('action', char.empty, 'replace', replaceStruct);
    end
    function input = collectFilterTypeInfo(input, filterTypeName)
      input.filterType = filterTypeName;
      switch filterTypeName
        case 'Whole Molecule'
          input.threshInfo.TF = false;
        case 'Threshold'
          input.threshInfo.TF = true;
      end
    end
    function input = collectDataToUseInfo(input,dataToUseDropDown)
      switch dataToUseDropDown.Visible
        case true
          input.dataToUse = dataToUseDropDown.Value;
        case false
          input.dataToUse = 'all';
      end
    end
    function input = collectActionObjectData(input,actionGrid)
      actionObjects = findobj(actionGrid.Children, '-not', 'Type', 'uilabel');
      for j = 1:length(actionObjects)
        switch actionObjects(j).Type
          case 'uicheckbox'
            input.actionToTake.replace.replacementValueSpecified = actionObjects(j).Value;
          case 'uinumericeditfield'
            input.actionToTake.replace.(actionObjects(j).Tag) = actionObjects(j).Value; %tag is either donor or acceptor
          case 'uidropdown'
            input.actionToTake.action = actionObjects(j).Value;
        end
      end
    end
    function input = collectConditionObjectData(input,conditionsGrid)
      conditionObjects = findobj(conditionsGrid.Children, '-not', 'Type', 'uilabel');
      for j = 1:length(conditionObjects)
        switch conditionObjects(j).Type
          case 'uicheckbox'
            switch conditionObjects(j).Text
              case 'Conditions'
                input.addConditions.TF = conditionObjects(j).Value;
              otherwise %otherwise it is a checkbox saying whether or not to include red frames in condition
                input.addConditions.excludeRedFrames = conditionObjects(j).Value;
            end
          case 'uidropdown'
            input.addConditions.condition = conditionObjects(j).Value;
          case 'uispinner'
            input.addConditions.minOrMax = conditionObjects(j).Tag;
            input.addConditions.lengthRequirement = conditionObjects(j).Value;
        end
      end
    end
    function input = collectRedFrameObjectData(input,redFrameObject)
      switch redFrameObject.Type
        case 'uispinner'
          input.redFramesRequirements.laserSwitch = redFrameObject.Value;
        case 'uinumericeditfield'
          input.redFramesRequirements.(redFrameObject.UserData) = redFrameObject.Value;
        case 'uigridlayout'
          redFrameObjects = redFrameObject.Children;
          for j = 1:length(redFrameObjects)
            switch redFrameObjects(j).Type
              case 'uicheckbox'
                switch redFrameObjects(j).Text
                  case 'Only Apply If Red Frames Are NOT Bleached'
                    input.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached = redFrameObjects(j).Value;
                  case 'Only Apply If Red Frames Are Bleached'
                    input.redFramesRequirements.onlyApplyIfRedFramesAreBleached = redFrameObjects(j).Value;
                end
              case 'uinumericeditfield'
                input.redFramesRequirements.redFramesThresh = redFrameObjects(j).Value;
            end
          end
      end
    end
    function input = collectSmoothObjectData(input,smoothGrid)
      smoothDataObjects = smoothGrid.Children;
      for j = 1:length(smoothDataObjects)
        switch smoothDataObjects(j).Type
          case 'uicheckbox'
            input.smoothData.TF = smoothDataObjects(j).Value;
          case 'uidropdown'
            input.smoothData.smoothType = smoothDataObjects(j).Value;
          case 'uispinner'
            input.smoothData.windowSize = smoothDataObjects(j).Value;
        end
      end
    end
    function input = collectPaddingObjectData(input,paddingGrid)
      paddingObjects = paddingGrid.Children;
      for j = 1:length(paddingObjects)
        switch paddingObjects(j).Type
          case 'uicheckbox'
            input.addPadding.TF = paddingObjects(j).Value;
          case 'uispinner'
            input.addPadding.amount = paddingObjects(j).Value;
        end
      end
    end
    function input = collectThresholdObjectData(input,threshGrid)
      threshObjects = findobj(threshGrid.Children, 'Type', 'uinumericeditfield');
      input.threshInfo.numThresholds = 0;
      for j = 1:length(threshObjects)
        switch threshObjects(j).Visible
          case true
            input.threshInfo.(threshObjects(j).Tag).(threshObjects(j).UserData) = threshObjects(j).Value;
            input.threshInfo.numThresholds = input.threshInfo.numThresholds +1;
        end
      end
    end
    function input = determineIfRedFramesAreRequired(input)
      if input.addConditions.excludeRedFrames == true || input.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
        input.redFramesRequirements.TF = true;
      elseif input.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
        input.redFramesRequirements.TF = true;
      else
        input.redFramesRequirements.TF = false;
      end
    end
    function input = gatherPlotInfo(input)
      if input.smoothData.TF == true || ismember(input.dataToUse, {'Sum Acceptor Donor'})
        input.thirdAxForPlot = true;
      end
    end
    function corn = createFilterDescriptionText(input, whichMolsText)
      corn{1} = "Data Filter Applied";
      corn{2} = whichMolsText;
      corn{3} = "Filter Type: " + string(input.filterType);
      switch input.filterType
        case 'Whole Molecule'
          if input.addConditions.TF == true
            conditionsText = getConditionsText(input);
            corn{end+1} = conditionsText;
          end
          if input.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true || input.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
            redFrameText = getRedFrameText(input);
            corn{end+1} = redFrameText;
          end
          if input.smoothData.TF == true
            smoothText = getSmoothText(input);
            corn{end+1} = smoothText;
          end
        case 'Threshold'
          corn{end+1} = "Data Used: " + string(input.dataToUse);
          threshText = getThreshText(input);
          corn{end+1} = threshText;
          smoothText = getSmoothText(input);
          corn{end+1} = smoothText;
          
          if input.addConditions.TF == true
            conditionsText = getConditionsText(input);
            corn{end+1} = conditionsText;
          end
          if input.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true || input.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
            redFrameText = getRedFrameText(input);
            corn{end+1} = redFrameText;
          end
          if input.addPadding.TF == true
            corn{end+1} = "Padding: " + num2str(input.addPadding.amount);
          end
        corn{end+1} = "Action: " + string(input.actionToTake.action);
        switch input.actionToTake.action
          case 'Replace'
            if input.actionToTake.replace.replacementValueSpecified == true
              corn{end+1} = "Replacement Method: Values Specified";
              corn{end+1} = "Donor: " + num2str(input.actionToTake.replace.donor) + ", Acceptor: " + num2str(input.actionToTake.replace.acceptor);
            else
              corn{end+1} = "Replacement Method: Nearest Points Averaged";
            end
        end
      end
    % embedded functions:
      function redFrameText = getRedFrameText(input)
        if input.redFramesRequirements.onlyApplyIfRedFramesAreBleached == true
          redFrameText = ["Condition Added: Only Apply if Red Laser Frames Are Bleached"; ...
            "Threshold For Red Frames Bleached: " + num2str(input.redFramesRequirements.redFramesThresh);...
            "Green Start Frame: " + num2str(input.redFramesRequirements.greenStartFrame); ...
            "Green End Frame: " + num2str(input.redFramesRequirements.greenEndFrame); ...
            "Number of laser switch frames: " + num2str(input.redFramesRequirements.laserSwitch)];
          
        elseif input.redFramesRequirements.onlyApplyIfRedFramesAreNOTBleached == true
          redFrameText = ["Condition Added: Only Apply if Red Laser Frames Are Not Bleached"; ...
            "Threshold For Red Frames Bleached: " + num2str(input.redFramesRequirements.redFramesThresh);...
            "Green Start Frame: " + num2str(input.redFramesRequirements.greenStartFrame); ...
            "Green End Frame: " + num2str(input.redFramesRequirements.greenEndFrame); ...
            "Number of laser switch frames: " + num2str(input.redFramesRequirements.laserSwitch)];
        end
        
      end
      function conditionsText = getConditionsText(input)
        switch input.addConditions.condition
          case {'Sections Before and After are Bleached', 'Section is at Begining or End'}
            conditionsText = ["Condition Added: " + string(input.addConditions.condition); ...
              "Red Laser Frames Excluded in condition: " + input.addConditions.excludeRedFrames];
          otherwise
            m = input.addConditions.minOrMax;
            m(1) = 'M';
            conditionsText = "Condition Added: " + string(m) + "Section Length:" + num2str(input.addConditions.lengthRequirement);
        end
      end
      function smoothText = getSmoothText(input)
        switch input.smoothData.TF
          case true
            smoothText = ["Smoothed: true"; "Type: " + string(input.smoothData.smoothType); ...
              "Window Size: " + num2str(input.smoothData.windowSize)];
          case false
            smoothText = "Smoothed: false";
        end
      end
      function threshText = getThreshText(input)
        threshText = string.empty;
        switch input.dataToUse
          case {'Donor AND Acceptor', 'Donor OR Acceptor'}
            minOrMaxText = fieldnames(input.threshInfo.bottom);
            threshText = "Acceptor Thresholds: ";
            tempText = string.empty;
            for i = 1:size(minOrMaxText,1)
              minMax = minOrMaxText{i};
              minMax(1) = 'M';
              tempText = tempText + string(minMax) + ": " + num2str(input.threshInfo.bottom.(minOrMaxText{i}));
              if size(minOrMaxText,1) == 2 && i == 1
                tempText = tempText + ", ";
              end
            end
            threshText(end+1) = tempText;
            threshText(end+1) = "Donor Thresholds: ";
            tempText = string.empty;
            minOrMaxText = fieldnames(input.threshInfo.top);
            for i = 1:size(minOrMaxText,1)
              minMax = minOrMaxText{i};
              minMax(1) = 'M';
              tempText = tempText + string(minMax) + ": " + num2str(input.threshInfo.top.(minOrMaxText{i}));
              if size(minOrMaxText,1) == 2 && i == 1
                tempText = tempText + ", ";
              end
            end
            threshText(end+1) = tempText;
          otherwise
            minOrMaxText = fieldnames(input.threshInfo.top);
            for i = 1:size(minOrMaxText,1)
              minMax = minOrMaxText{i};
              minMax(1) = 'M';
              threshText(end+1) = string(minMax) + " Threshold: " + num2str(input.threshInfo.top.(minOrMaxText{i})); %#ok
            end
        end
      end %end of getThreshText fcn
    end
  end
  methods (Sealed = true)

  end
  
end

