classdef kmeansResults %< handle
  properties
    k
    centerInitialization %not currently in use - eventually add different methods for initializing centers
    methodForMeasuringDistance
    whichTracesToRun %{batches, onlyThisOne, oneAtATime, all}
    maxNumLoops 
    numTracesPerBatch
    whichDataToCluster
    specificDataToOmit
    results
    stats
  end
  methods
    %constructor function
    function kmeansObj = kmeansResults(handleToAllData,valuesFromApp, progressBar)
      varNames = fields(valuesFromApp);
      for i = 1:size(varNames,1)
        kmeansObj.(varNames{i}) = valuesFromApp.(varNames{i});
      end
      whatData = kmeansObj.whichTracesToRun;
      whatData.dataNames = kmeansObj.whichDataToCluster;



      dataMatrix = kmeansObj.makeDataMatrix(handleToAllData, kmeansObj.whichTracesToRun, kmeansObj.whichDataToCluster);
      %kmeansObj.centerInitialization.mode = "manual";
      kmeansObj.centerInitialization.mode = "Notmanual";
      if kmeansObj.centerInitialization.mode ~= "manual"
      initialCenters = kmeansObj.initializeCenters(kmeansObj.k, kmeansObj.whichDataToCluster , dataMatrix);
      else
        c = [0, 700, 200];
        for w = 1:3
          fieldName = "centers" + num2str(w);
          initialCenters.(fieldName) = c(w);
        end
        
      end
      [distFcn, newCentersFcn] = kmeansObj.setDistanceAndCenterFunctions(kmeansObj.methodForMeasuringDistance, kmeansObj.whichDataToCluster);
      [clusterIndex, clusterCenters] = kmeansObj.runKmeansClusteringAlgorithm(dataMatrix, ...
        initialCenters, kmeansObj.k, kmeansObj.maxNumLoops, distFcn, newCentersFcn, progressBar);
      
      
      
      % calculate some stats
      dataNames = {'donor', 'acceptor', 'fret'};
      for i = 1:3
        [dataPoints.(dataNames{i}), ~] = kmeansObj.stitchDataTogether(handleToAllData, kmeansObj.whichTracesToRun, dataNames{i});
      end

      [averages, adResults, kmeansObj.stats] = kmeansObj.calculateClusterStats(clusterIndex, dataPoints, kmeansObj.k);
      
      kmeansObj.results.clusterIndex = clusterIndex;
      
      kmeansObj.results.clusterCenters = clusterCenters;
      kmeansObj.results.averages = averages;
      kmeansObj.results.adTest = adResults;
    end

    function dataMatrix = makeDataMatrix(kmeansObj, handleToAllData, whichData)
      switch whichData.dataNames %kmeansObj.whichDataToCluster %handleToApp.DataToClusterDropDown.Value;
        %returns array of data to cluster with fret data attached in last
        %column
        case 'Donor & Acceptor'
          [donorData, molAndTraceInfoColumns] = kmeansObj.stitchDataTogether(handleToAllData, whichTraces, 'donor');
          [acceptorData, ~] = kmeansObj.stitchDataTogether(handleToAllData, whichTraces, 'acceptor');
          [fretData, ~] = kmeansObj.stitchDataTogether(handleToAllData, whichTraces, 'fret');
          dataMatrix = [donorData, acceptorData, fretData, molAndTraceInfoColumns];
          %           dataFcn = @(traceNum, molNum) [handleToAllData.traces{traceNum}.molecules{molNum}.currentData.donor, ...
          %             handleToAllData.traces{traceNum}.molecules{molNum}.currentData.acceptor, ...
          %             handleToAllData.traces{traceNum}.molecules{molNum}.currentData.fret];
          %returns #frames x 3 array
        case {'Fret', 'Donor', 'Acceptor'}
          whichData = lower(whichData);
          [data, molAndTraceInfoColumns] = kmeansObj.stitchDataTogether(handleToAllData, whichTraces, whichData);
          [fretData, ~] = kmeansObj.stitchDataTogether(handleToAllData, whichTraces, 'fret');
          dataMatrix = [data, fretData, molAndTraceInfoColumns];
          %           dataFcn = @(traceNum, molNum) [handleToAllData.traces{traceNum}.molecules{molNum}.currentData.(whichData), ...
          %             handleToAllData.traces{traceNum}.molecules{molNum}.currentData.fret];
      end
    end
    
  end

  methods (Static)
    
    function varargout = stitchData(whichTraces, whichData, whichMolecules, molArray, numDataPoints)
            arguments
        whichTraces %includes mol numbers, method, and which data names
        whichData
        numFramesTotal
        moleculeArray
        options.totalNumMols
        options.molNum
            end
      varargout = {};
      dataFcn = @(molNum) moleculeArray(molNum).currentData.(whichData);
      switch whichTraces.method %kmeansObj.whichTracesToRun %handleToApp.HowtorunthetracesButtonGroup.SelectedObject.Tag;
        case 'all'
          %input needed: totalNumDataPoints, totalNumMols, moleculeArray
          stitchedData = nan(numFramesTotal, 1);
          molAndTraceInfoColumns = nan(numFramesTotal, 2);
          nMols = totalNumMols;
          i = 1; idx = 1;
          while i <= nMols
            data = dataFcn(i);
            numFramesThisMol = size(data,1);
            idxEnd = idx + numFramesThisMol - 1;
            stitchedData(idx:idxEnd,:) = data;
            traceNum = moleculeArray(i).traceNum;
            molNum = moleculeArray(i).moleculeNumber;
            molAndTraceInfoColumns(idx:idxEnd,:) = [ones(numFramesThisMol,1)*molNum, ones(numFramesThisMol,1)*traceNum];
            idx = idxEnd + 1;
            i = i+1;
          end

        case 'onlyThisMol'
          %input needed: molNumber, moleculeArray
          nFrames = moleculeArray.numFrames;
          molAndTraceInfoColumns = [ones(nFrames,1)*whichTraces.molNum, ones(nFrames,1)*whichTraces.traceNum];
          stitchedData = dataFcn(whichTraces.traceNum, whichTraces.molNum);
        case {'onlyThisFile', 'oneFileAtATime'}
          %
          nFrames = handleToApp.traces{whichTraces.traceNum}.totalNumDataPointsThisTrace;
          nMols = handleToApp.traces{whichTraces.traceNum}.numMolecules;
          stitchedData = nan(nFrames, 1);
          molAndTraceInfoColumns = [nan(nFrames,1), ones(nFrames,1)*whichTraces.traceNum];
          j = 1; idx = 1;
          while j <= nMols
            data = dataFcn(whichTraces.traceNum,j);
            numFramesThisMol = size(data,1);
            idxEnd = idx + numFramesThisMol - 1;
            stitchedData(idx:idxEnd,:) = data;
            molAndTraceInfoColumns(idx:idxEnd,1) = ones(numFramesThisMol,1)*j;
            idx = idxEnd + 1;
            j = j+1;
          end
          % if it's one file at a time, then need to update
          % whichTraces.traceNum after each loop
        case 'batches'
      end
    end
    
    

    function [stitchedData, molAndTraceInfoColumns] = stitchDataTogether(whichTraces, whichData, numFramesTotal, moleculeArray, options)
      arguments
        whichTraces %includes mol numbers, method, and which data names
        whichData
        numFramesTotal
        moleculeArray
        options.totalNumMols
        options.molNum
      end
      dataFcn = @(molNum) moleculeArray(molNum).currentData.(whichData);
      switch whichTraces.method %kmeansObj.whichTracesToRun %handleToApp.HowtorunthetracesButtonGroup.SelectedObject.Tag;
        case 'all'
          %input needed: totalNumDataPoints, totalNumMols, moleculeArray
          stitchedData = nan(numFramesTotal, 1);
          molAndTraceInfoColumns = nan(numFramesTotal, 2);
          nMols = totalNumMols;
          i = 1; idx = 1;
          while i <= nMols
            data = dataFcn(i);
            numFramesThisMol = size(data,1);
            idxEnd = idx + numFramesThisMol - 1;
            stitchedData(idx:idxEnd,:) = data;
            traceNum = moleculeArray(i).traceNum;
            molNum = moleculeArray(i).moleculeNumber;
            molAndTraceInfoColumns(idx:idxEnd,:) = [ones(numFramesThisMol,1)*molNum, ones(numFramesThisMol,1)*traceNum];
            idx = idxEnd + 1;
            i = i+1;
          end

        case 'onlyThisMol'
          %input needed: molNumber, moleculeArray
          nFrames = moleculeArray.numFrames;
          molAndTraceInfoColumns = [ones(nFrames,1)*whichTraces.molNum, ones(nFrames,1)*whichTraces.traceNum];
          stitchedData = dataFcn(whichTraces.traceNum, whichTraces.molNum);
        case {'onlyThisFile', 'oneFileAtATime'}
          %
          nFrames = handleToApp.traces{whichTraces.traceNum}.totalNumDataPointsThisTrace;
          nMols = handleToApp.traces{whichTraces.traceNum}.numMolecules;
          stitchedData = nan(nFrames, 1);
          molAndTraceInfoColumns = [nan(nFrames,1), ones(nFrames,1)*whichTraces.traceNum];
          j = 1; idx = 1;
          while j <= nMols
            data = dataFcn(whichTraces.traceNum,j);
            numFramesThisMol = size(data,1);
            idxEnd = idx + numFramesThisMol - 1;
            stitchedData(idx:idxEnd,:) = data;
            molAndTraceInfoColumns(idx:idxEnd,1) = ones(numFramesThisMol,1)*j;
            idx = idxEnd + 1;
            j = j+1;
          end
          % if it's one file at a time, then need to update
          % whichTraces.traceNum after each loop
        case 'batches'
      end
    end

    function initialCenters = initializeCenters(k, whichData, dataMatrix)
      % remove 'nan' points
      dataMatrix = dataMatrix(~isnan(dataMatrix(:,1)),:);
      switch whichData
        case {'Fret', 'Donor', 'Acceptor'}
          % get data points and then clear the data matrix
          dataPoints = dataMatrix(:,1);
          fretPoints = dataMatrix(:,2);
          clear dataMatrix
          %%% sort data and remove duplicate points
          %%dataPoints = unique(dataPoints);
          [~, ia] = sort(fretPoints);
          sortedFret = fretPoints(ia);
          sortedData = dataPoints(ia);
          %dataPoints = sort(dataPoints);
          % return numpoints and initial centers fcn, and clear dataPoints
          npoints = length(dataPoints);
          initialCentersFcn = @(randomIdxNum) dataPoints(randomIdxNum);
          clear dataPoints  %they are still stored in the function above
        case 'Donor & Acceptor'
          % use the sum as the sorting data
          dataPoints = dataMatrix(:,1) + dataMatrix(:,2);
          % or use fret
          dataPoints = dataMatrix(:,3);
          %%% sort data and remove duplicate points
          %%[~, ia, ~] = unique(dataPoints);
          [~, ia] = sort(dataPoints);
          clear dataPoints
          % sort original data using index from sorted data
          donorData = dataMatrix(ia,1);
          acceptorData = dataMatrix(ia,2);
          sortedFret = dataMatrix(ia,3);
          
          clear dataMatrix ia
          npoints = length(acceptorData);
          % make function handle to use below
          initialCentersFcn = @(randomIdxNum) [donorData(randomIdxNum), acceptorData(randomIdxNum)];
          clear donorData acceptorData  %they are still stored in the function above
      end
      lowFretIdxEnd = find(sortedFret <= -2,1,'last');
      highFretIdxStart = find(sortedFret > 2, 1);
      w = [~isempty(lowFretIdxEnd), ~isempty(highFretIdxStart)];
      idxStart = 1; idxEnd = npoints; kStart = 1; kEnd = k;
      if any(w)
        if ~isempty(lowFretIdxEnd)
          idxStart = lowFretIdxEnd+1;
        end
        if ~isempty(highFretIdxStart)
          idxEnd = highFretIdxStart-1;
        end
      end
      if k ~= 2 && any(w)
        if all(w == true)
          firstInterval = [1,idxStart];
          lastInterval = [(idxEnd+1), npoints];
          npoints = length(idxStart:idxEnd);
          kStart = 2; kEnd = k-1;
        elseif w(1) == true
          firstInterval = [1,idxStart];
          npoints = npoints - length(1:idxStart);
          kStart = 2;
        elseif w(2) == true
          lastInterval = [(idxEnd+1), npoints];
          npoints = npoints - length((idxEnd+1):npoints);
          kEnd = k-1;
        end
      elseif any(w) && ~all(w)
        if w(1) == true
          firstInterval = [1,idxStart];
          npoints = npoints - length(1:idxStart);
          kStart = 2;
        elseif w(2) == true
          lastInterval = [(idxEnd+1), npoints];
          npoints = npoints - length((idxEnd+1):npoints);
          kEnd = k-1;
        end
      else
        idxStart = 1;
      end
      % divide the data into k sections and pick a random point from each
      % section to be a starting center
      lengthOfSection = floor(npoints/k);
      for j = kStart:kEnd
        intervalFromWhichToChooseCenter = [(idxStart+lengthOfSection*(j-1)), lengthOfSection*j];
        indexChosenInInterval = randi(intervalFromWhichToChooseCenter, 1, 1);
        fieldName = "centers" + num2str(j);
        initialCenters.(fieldName) = initialCentersFcn(indexChosenInInterval);
      end
      if kStart ~=1
        indexChosenInInterval = randi(firstInterval, 1, 1);
        initialCenters.centers1 = initialCentersFcn(indexChosenInInterval);
      end
      if kEnd ~= k
        indexChosenInInterval = randi(lastInterval, 1, 1);
        fieldName = "centers" + num2str(k);
        initialCenters.(fieldName) = initialCentersFcn(indexChosenInInterval);
      end
    end
    function [distFcn, newCentersFcn] = setDistanceAndCenterFunctions(methodForMeasuringDistance, whichDataToCluster)
      switch whichDataToCluster
        case 'Donor & Acceptor'
          newCentersFcn = @(datapoints) [mean(datapoints(:,1)), mean(datapoints(:,2))];
        otherwise
          newCentersFcn = @(datapoints) mean(datapoints);
      end
      % note, check out this page for other cool distance methods:
      %https://www.mathworks.com/help/stats/pdist2.html
      switch methodForMeasuringDistance
        case 'Euclidean'
          distFcn = @(columnsOfData, centersToMeasureTo) pdist2(columnsOfData, centersToMeasureTo);
        case 'Mahalanobis'
          distFcn = @(columnsOfData, centersToMeasureTo) pdist2(columnsOfData, centersToMeasureTo, 'mahalanobis');
        case 'MahalanobisGMM'
          disp('add code for other distance types in kmeansResults.m');
        case 'Taxicab'
          distFcn = @(columnsOfData, centersToMeasureTo) pdist2(columnsOfData, centersToMeasureTo, 'cityblock');
        case 'Hamming'
          % coords must be exact for this one - useless
          distFcn = @(columnsOfData, centersToMeasureTo) pdist2(columnsOfData, centersToMeasureTo, 'hamming');
          %Hamming distance, which is the percentage of coordinates that differ.
        case 'Chebychev'
          distFcn = @(columnsOfData, centersToMeasureTo) pdist2(columnsOfData, centersToMeasureTo, 'chebychev');
          % (maximum coordinate difference)- not very useful..
      end
    end
     
    function [clusterIndex, clusterCenters] = runKmeansClusteringAlgorithm(dataMatrix, initialCenters,...
        k, maxNumLoops, distFcn, newCentersFcn, progressBar)
      %dataMatrix: array with columns: datapoints (1 or 2 columns), fretData, molNum, traceNum
      numPoints = size(dataMatrix,1);
      numDataColumns = size(dataMatrix,2) - 3; %3 = fretData, mol & trace num
      data = dataMatrix(:,1:numDataColumns);
      fretDataColumn = size(dataMatrix,2) - 2;
      fretData = dataMatrix(:,fretDataColumn);
      clusterIndex = zeros(numPoints, k+2);
      % assign trace and mol number to last two columns of clusterIndex
      clusterIndex(:,(end-1):end) = dataMatrix(:,(end-1):end);
      clear dataMatrix
      %clusterIndex = zeros(numPoints,k+2);
      %clusterIndex(:,(k+1):(k+2)) = molAndTraceId;
      clusterIndexLastLoop = clusterIndex;
      clusterCenters = initialCenters;
      centerNames = fields(clusterCenters);
      %clear dataMatrix numDataColumns molAndTraceId initialCenters
      loop = 0; keepGoing = true;
      while keepGoing == true
        loop = loop + 1;
        progressBar.Message = "Running Kmeans Algorithm for " + num2str(k) + " Clusters. Loop " + num2str(loop);
        distances = zeros(numPoints, k);
        for i = 1:k
          distances(:,i) = distFcn(data, clusterCenters.(centerNames{i}));
        end
        %Note: add something for if point is equidistant to two centers
        % calculate min distance to each center. idxOfMinValues gives
        % cluster number
        [minValues, idxOfMinValues] = min(distances, [], 2);
        %clear distances
        % set nan coords back to nan in index - otherwise they are 1 in idx
        idxOfMinValues(isnan(minValues)== true) = nan;
        % create logical index for each cluster and calculate new center
        for i = 1:k
          clusterIndex(:,i) = idxOfMinValues == i;
          dataPointsThisCluster = data(idxOfMinValues == i,:);
          clusterCenters.(centerNames{i}) = newCentersFcn(dataPointsThisCluster);
        end
        if all(clusterIndex == clusterIndexLastLoop, 'all') || loop == maxNumLoops
          keepGoing = false;
          % calculate mean of fret for each cluster
          clusterCenters.fretCenters = zeros(k,1);
          for i = 1:k
            fretDataPointsThisCluster = fretData(clusterIndex(:,i) == 1);
            %fretDataPointsThisCluster = dataMatrix(clusterIndex(:,i) == 1,fretDataColumn);
            clusterCenters.fretCenters(i) = mean(fretDataPointsThisCluster);
          end
        else
          clusterIndexLastLoop = clusterIndex;
        end
      end %end of while loop
    end

    function [averages, adResults, stats] = calculateClusterStats(clusterIndex, data, k)
      dataNames = {'donor', 'acceptor', 'fret'};
      %data.donor = donor; data.acceptor = acceptor; data.fret = fret;
      for i = 1:3
        adInfo = cell(k+1,6);
        adInfo(1,:) = {'center','h','p','adstat','cv', 'nPoints'};
        adInfo_normalDist = adInfo;
        adInfo_extremeValueDist = adInfo;
        for j = 1:k
          clusterFieldname = "cluster" + num2str(j);
          allData = data.(dataNames{i});
          clusterJData = allData(clusterIndex(:,j) == 1);
          stats.(clusterFieldname).(dataNames{i}).mean = mean(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).median = median(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).std = std(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).min = min(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).max = max(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).mode = mode(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).iqr = iqr(clusterJData);
          % iqr(x) = the difference between the 75th and the 25th percentiles of the data contained in x.
          stats.(clusterFieldname).(dataNames{i}).meanAbsoluteDeviation = mad(clusterJData);
          stats.(clusterFieldname).(dataNames{i}).medianAbsoluteDeviation = mad(clusterJData,1);
          if length(clusterJData) > 10
          stats.(clusterFieldname).(dataNames{i}).gaussFit = fitdist(clusterJData,'Normal');
          [phat,pci] = mle(clusterJData);
          %returns maximum likelihood estimates (MLEs) for the parameters of a normal distribution
          stats.(clusterFieldname).(dataNames{i}).mle_phat = phat;
          stats.(clusterFieldname).(dataNames{i}).mle_pci = pci;
          %%% everything below is a work in progress - trying to find the
          %%% best way to estimate k
          averages.(clusterFieldname).(dataNames{i}).avg = mean(clusterJData);
          averages.(clusterFieldname).(dataNames{i}).sd = std(clusterJData);
          dist = makedist('normal','mu',averages.(clusterFieldname).(dataNames{i}).avg,'sigma',averages.(clusterFieldname).(dataNames{i}).sd);
          [h,p,adstat,cv] = adtest(clusterJData, 'Distribution',dist);
          %to use monetCarlo p value instead, 'MCTol',0.01,
          % also try 'Distribution','ev')  'Alpha',0.01
          averages.(clusterFieldname).(dataNames{i}).h = h;
          averages.(clusterFieldname).(dataNames{i}).p = p;
          averages.(clusterFieldname).(dataNames{i}).adstat = adstat;
          averages.(clusterFieldname).(dataNames{i}).cv = cv;
          adInfo_normalDist(j+1,:) = {averages.(clusterFieldname).(dataNames{i}).avg ,h,p,adstat,cv, length(clusterJData)};
          [h,p,adstat,cv] = adtest(clusterJData, 'Distribution','ev');
          adInfo_extremeValueDist(j+1,:) = {averages.(clusterFieldname).(dataNames{i}).avg, h,p,adstat,cv, length(clusterJData)};
          end
        end
        adResults.(dataNames{i}).normalDist = adInfo_normalDist;
        adResults.(dataNames{i}).extremeValueDist = adInfo_extremeValueDist;
        
      end
    end
  end
end


