function [data, numDataPoints] = loadTraceData(filenames)
numFiles = size(filenames,2); %filenames should be a cell with char names
data = struct('donor', {}, 'acceptor', {}, 'fret', {}, 'traceNum', {}, 'traceName', {});
numDataPoints = 0;

for i = 1:numFiles
  fname = filenames{i};
  % open and read in the file information
  fid = fopen(fname,'r');
  numFrames = fread(fid,1,'int32'); % len1gth in frames
  Ntraces = fread(fid,1,'int16'); % number of traces, # molecules is Ntraces/2
  numMolecules = Ntraces/2;
  raw = fread(fid,(Ntraces+1)*numFrames,'int16');
  fclose(fid);
  % reshape data so each column is the data from one frame
  Data = reshape(raw,[],numFrames);
  % first row of Data are frame numbers
  Data = Data(2:end, :);
  % rest of Data rows alternate between acceptor and donor
  acceptorData = Data(1:2:end, :);
  donorData = Data(2:2:end, :);
  for m = 1:numMolecules
    idx = size(data,2) + 1;
    data(idx).donor = donorData(m,:)';
    data(idx).acceptor = acceptorData(m,:)';
    data(idx).fret = (data(idx).acceptor)./(data(idx).acceptor + data(idx).donor);
    data(idx).traceNum = i;
    data(idx).traceName = fname;
    numDataPoints = numDataPoints + length(data(idx).donor);
  end
end
end