    function [clusterIndex, clusterCenters] = runKmeansClusteringAlgorithm(dataMatrix, initialCenters,...
        k, maxNumLoops, distFcn, newCentersFcn, progressBar)
      %dataMatrix: array with columns: datapoints (1 or 2 columns), fretData, molNum, traceNum
      numPoints = size(dataMatrix,1);
      numDataColumns = size(dataMatrix,2) - 3; %3 = fretData, mol & trace num
      data = dataMatrix(:,1:numDataColumns);
      fretDataColumn = size(dataMatrix,2) - 2;
      fretData = dataMatrix(:,fretDataColumn);
      clusterIndex = zeros(numPoints, k+2);
      % assign trace and mol number to last two columns of clusterIndex
      clusterIndex(:,(end-1):end) = dataMatrix(:,(end-1):end);
      clear dataMatrix
      %clusterIndex = zeros(numPoints,k+2);
      %clusterIndex(:,(k+1):(k+2)) = molAndTraceId;
      clusterIndexLastLoop = clusterIndex;
      clusterCenters = initialCenters;
      disp('Initial Centers:');
      disp(initialCenters);
      temp = [];
      centerNames = fields(clusterCenters);
      loop = 0; keepGoing = true;
      while keepGoing == true
        loop = loop + 1;
        progressBar.Message = "Running Kmeans Algorithm for " + num2str(k) + " Clusters. Loop " + num2str(loop);
        distances = zeros(numPoints, k);
        for i = 1:k
          distances(:,i) = distFcn(data, clusterCenters.(centerNames{i}));
        end
        %Note: add something for if point is equidistant to two centers
        % calculate min distance to each center. idxOfMinValues gives
        % cluster number
        [minValues, idxOfMinValues] = min(distances, [], 2);

        clear distances
        % set nan coords back to nan in index - otherwise they are 1 in idx
        idxOfMinValues(isnan(minValues)== true) = nan;
   
        

        % create logical index for each cluster and calculate new center
        for i = 1:k
          clusterIndex(:,i) = idxOfMinValues == i;
          dataPointsThisCluster = data(idxOfMinValues == i,:);
          clusterCenters.(centerNames{i}) = newCentersFcn(dataPointsThisCluster);
        end
        if all(clusterIndex == clusterIndexLastLoop, 'all') || loop == maxNumLoops
          keepGoing = false;
          % calculate mean of fret for each cluster
          clusterCenters.fretCenters = zeros(k,1);
          for i = 1:k
            fretDataPointsThisCluster = fretData(clusterIndex(:,i) == 1);
            %fretDataPointsThisCluster = dataMatrix(clusterIndex(:,i) == 1,fretDataColumn);
            clusterCenters.fretCenters(i) = mean(fretDataPointsThisCluster);
          end
                disp('Final FRET Centers:');
             disp(clusterCenters.fretCenters);
        else
          clusterIndexLastLoop = clusterIndex;
        end
      end %end of while loop
    end