function saveDwellTimesAsIgorFile(dwellTimes, fileNameToSaveAs, framerate, expName)
arguments
  dwellTimes
  fileNameToSaveAs
  framerate
  expName = char.empty;
end

%dwells.center2.dwellTimes = nx1 array of dwell times
waveNames = fieldnames(dwellTimes);
if isfield(dwellTimes.(waveNames{1}), 'value')
  q = 1; %then it's normal dwell times. Otherwise it's transition times
else
  q = 2;
end
nWaves = size(waveNames,1);
histNames = cell(size(waveNames));
numBins = zeros(size(waveNames)); binStart = zeros(size(waveNames));
binEnd = zeros(size(waveNames));
binWidth = framerate*4/1000;
formatSpecForData = char.empty;
notebookTextLines = [];
textBoxLinesForGraph = cell(size(waveNames));
maxDataArrayLength = 1;
waveDeclarations = 'WAVES/D';
for i = 1:nWaves
  waveDeclarations = [waveDeclarations, ' ', waveNames{i}, ','];
  formatSpecForData = [formatSpecForData, '%i '];
  %add lines to create a notebook showing what fret level goes with which
  %wave name
  switch q
    case 1
      notebookTextLines = [notebookTextLines, string(waveNames{i}) + " FRET: " + num2str(dwellTimes.(waveNames{i}).value) + "\r "];
      textBoxLinesForGraph{i} = ['TextBox/C/N=text0/F=0/A=MT/X=0.00/Y=0.00/E "\\Z14\\f05', expName, ...
    ' Dwell Times: FRET Efficiency ',num2str(dwellTimes.(waveNames{i}).value), '\r\\Z12\\f07\\JC\\f02', 'time resolution = ',...
    num2str(framerate), 'ms  n = ', num2str(dwellTimes.(waveNames{i}).idx - 1), '"'];
  
    case 2
      notebookTextLines = [notebookTextLines, string(waveNames{i}) + ": Transition from FRET " + num2str(dwellTimes.(waveNames{i}).values(1))...
        + " To FRET " + num2str(dwellTimes.(waveNames{i}).values(2)) +  "\r "];
      textBoxLinesForGraph{i} = ['TextBox/C/N=text0/F=0/A=MT/X=0.00/Y=0.00/E "\\Z14\\f05', expName, ...
    ' Dwell Times of Kinetic State Transition \r From FRET ',num2str(dwellTimes.(waveNames{i}).values(1)), ' to FRET ', ...
    num2str(dwellTimes.(waveNames{i}).values(2)),'\r\\Z12\\f07\\JC\\f02', 'time resolution = ',...
    num2str(framerate), 'ms  n = ', num2str(dwellTimes.(waveNames{i}).idx - 1), '"'];
  end
  % get info for histogram commands
  histNames{i} = [waveNames{i}, '_Hist'];
  numBins(i) = ceil((dwellTimes.(waveNames{i}).max - dwellTimes.(waveNames{i}).min)/binWidth);
  binStart(i)  = (dwellTimes.(waveNames{i}).min)-(framerate/(2*1000));
  binEnd(i) = binStart(i) + numBins(i)*binWidth;

  maxDataArrayLength = max(maxDataArrayLength, length(dwellTimes.(waveNames{i}).dwellTimes));
end
waveDeclarations = waveDeclarations(1:(end-1));
waveDeclarations = string(waveDeclarations);
firstThreeLines = ["IGOR", waveDeclarations,"BEGIN"];
formatSpecForData = [formatSpecForData, '\n'];

%make sure the data arrays are all the same length
dataToPrint = nan(maxDataArrayLength, nWaves);
for i = 1:nWaves
  data = dwellTimes.(waveNames{i}).dwellTimes;
  dataToPrint(1:length(data),i) = data;
end






fid = fopen(fileNameToSaveAs,'w');
fprintf(fid, '%s\n',firstThreeLines);
fprintf(fid, formatSpecForData, dataToPrint);
fprintf(fid,'END\n');


%add a notepad with details about the data
fprintf(fid,'\n');
noteBookName = ['ExperimentDetails', num2str(q)];
fprintf(fid,'%s\n', ['X NewNotebook/N=',noteBookName,'/F=0/K=0/W=(5.25,43.25,504.75,338)']);
fprintf(fid,'%s\n', ['X Notebook ',noteBookName,' defaultTab=20, statusWidth=252']);
fprintf(fid,'%s\n', ['X Notebook ',noteBookName,' font="Arial", fSize=10, fStyle=0, textRGB=(0,0,0)']);
fprintf(fid,'%s\n', ['X Notebook ',noteBookName,' text="Dwell Times \rUnits: seconds \rFrameRate: ', num2str(framerate), ' ms\r\r"']);
fprintf(fid, '%s', ['X Notebook ',noteBookName,' text="', notebookTextLines, '\r"']);
fprintf(fid,'\n');



% write commands to make histograms
for i = 1:nWaves
fprintf(fid, 'X %s\n', ['Make/N=', num2str(numBins(i)),'/O ', histNames{i}, ';DelayUpdate']);
fprintf(fid, 'X %s\n', ['Histogram/C/B={', num2str(binStart(i)),',', num2str(binWidth), ',', num2str(binEnd(i)),'} ', waveNames{i}, ',', histNames{i}, ';DelayUpdate']);
fprintf(fid, 'X %s\n', ['Display ', histNames{i}]);
fprintf(fid, 'X %s\n', 'Label left "\\Z14Count";DelayUpdate');
fprintf(fid, 'X %s\n', 'Label bottom "\\Z14Seconds"');
fprintf(fid, 'X %s\n',textBoxLinesForGraph{i});
end

fprintf(fid,'\n');


fclose(fid);
disp('saved igor');
disp(fileNameToSaveAs);
cd
binWidth = 10;
%saveTextFileForIgorCommands(dwellTimes, binWidth, 'copyAndPasteIgorCommands.txt');


end