function saveTextFileOfAnalysisDetails(history, saveName) %, dwellInfo, framerate)
%numberOfStepsTaken = size(history.stepsTaken,2) - 1;
fid = fopen(saveName,'wt');
%d = datestr(now, 'mm/dd/yyyy');
d = datestr(datetime('now'));
fprintf(fid, 'Analysis Date: ');
fprintf(fid, d);

if isfield(history, 'clusterCenters')
fprintf(fid, '\n\n%s', ['Number of FRET Efficiency States: ', num2str(size(history.clusterCenters,1))]);
fprintf(fid, '\n%s', 'Mean Values:');
for i = 1:size(history.clusterCenters,1)
  fprintf(fid, '\nFRET: %-5.2f  Acceptor: %-5.0f  Donor: %-5.0f', history.clusterCenters(i,:));
end
fprintf(fid, '\n\n');
end

fprintf(fid, '\n%s', ['Number of Molecules: ', num2str(history.numMolsBeforeProcessing)]);
fprintf(fid, '\n%s', ['Number of Molecules After Filtering: ', num2str(history.numMolsAfterProcessing)]);
fprintf(fid, '\n%s', ['Frame rate: ', char(num2str(history.frameRate)), ' ms']);
fprintf(fid, '\n\n%s\n\n', 'Analysis Steps: ');
history.fromApp
history.stepNamesFromApp
for w = 1:size(history.stepNamesFromApp,2)%i = 2:size(history.stepsTaken,2)
  stepName = history.fromApp{w}(1);
  linesToPrint = "Step " + num2str(w) + ": " + stepName;
  %linesToPrint = "Step " + num2str(i-1) + ":" + string(history.stepsTaken{i});
  formatSpec = '%s\n';
  switch history.stepNamesFromApp{w} 
    case {'postClustering', 'singleFramesAnalyzed', 'firstAndLastRemoved'}
      [~,i] = find(matches(history.stepsTaken, history.stepNamesFromApp{w}));
  if ~isempty(history.stepInfo{i})
    switch class(history.stepInfo{i})
      case 'struct'
        varNames = fieldnames(history.stepInfo{i});
        for j = 1:size(varNames,1)
          switch varNames{j}
            case 'centers'
              numCenters = length(history.stepInfo{i}.centers);%#ok
              for k = 1:length(history.stepInfo{i}.centers)
                formatSpec = [formatSpec,'%s\n'];%#ok
                linesToPrint(end+1) = ["FRET state " + num2str(k) + ": " + ...
                  string(history.stepInfo{i}.(varNames{j})(k))];%#ok
              end
            case 'stats'
              statFieldNames = fieldnames(history.stepInfo{i}.stats);
              nSections = history.stepInfo{i}.stats.numberOfSections;
              nDataPoints = history.stepInfo{i}.stats.numberOfDataPoints;
              [a,~] = find(strcmp(statFieldNames,'numberOfSections'));
              statFieldNames(a) = [];
              [a,~] = find(strcmp(statFieldNames,'numberOfDataPoints'));
              statFieldNames(a) = [];
              linesToPrintAtVeryEnd = ["Cluster Stats:"];
              formatSpecForVeryEnd = ['\n%s\n'];
              for h = 1:size(nSections,1)
                formatSpecForVeryEnd = [formatSpecForVeryEnd,'%s\n']; %#ok
                linesToPrintAtVeryEnd(end+1) = ["Cluster " + num2str(h) + ": " + ...
                  "(" + num2str(nSections(h)) + " sections, " + ...
                  num2str(nDataPoints(h)) + " data points total)"]; %#ok
                dataNames = {"FRET: ", "Acceptor: ", "Donor: "}; %#ok
                for w = 1:3
                  linesToPrintAtVeryEnd(end+1) = [dataNames{w}];%#ok
                  formatSpecForVeryEnd = [formatSpecForVeryEnd,'%s\n'];%#ok
                  for k = 1:size(statFieldNames,1)
                    formatSpecForVeryEnd = [formatSpecForVeryEnd,'%s\n'];%#ok
                  linesToPrintAtVeryEnd(end+1) = [convertCharsToStrings(statFieldNames{k}) + ": " ...
                    + num2str(round(history.stepInfo{i}.stats.(statFieldNames{k})(h,w),2))];%#ok
                  end
                end
                formatSpecForVeryEnd = [formatSpecForVeryEnd,'\n%s\n'];%#ok
                linesToPrintAtVeryEnd(end+1) = [" "];%#ok
              end
            otherwise
              formatSpec = [formatSpec,'%s\n'];%#ok
              switch class(history.stepInfo{i}.(varNames{j}))
                case 'char'
                  linesToPrint(end+1) = [convertCharsToStrings(varNames{j}) + ": " + ...
                    convertCharsToStrings(history.stepInfo{i}.(varNames{j}))];%#ok
                otherwise
                  linesToPrint(end+1) = [convertCharsToStrings(varNames{j}) + ": " ...
                    + string(history.stepInfo{i}.(varNames{j}))];%#ok
              end
          end
        end
    case 'char'
     linesToPrint(end+1) =  string(history.stepInfo{i});%#ok
     formatSpec = [formatSpec, '%s\n'];%#ok
    end
  end
    otherwise
      histThisStep = history.fromApp{w};
      switch class(histThisStep)
        case 'cell'
          for q = 2:length(histThisStep)
            oneCell = histThisStep{q};
            for t = 1:length(oneCell)
              linesToPrint(end+1) =  oneCell(t);%#ok
            formatSpec = [formatSpec, '%s\n'];%#ok
            end
          end
        case 'string'
          for q = 2:length(histThisStep)
            linesToPrint(end+1) =  histThisStep(q);%#ok
            formatSpec = [formatSpec, '%s\n'];%#ok
          end
      end

  end %end of loop through history.stepInfo 
  formatSpec = [formatSpec,'\n'];%#ok
  fprintf(fid, formatSpec, linesToPrint);
end %end of loop through number of steps taken
if exist('linesToPrintAtVeryEnd', 'var')
fprintf(fid, formatSpecForVeryEnd, linesToPrintAtVeryEnd);
end

fclose(fid);
end

